#################################################################################
#    Class::DBI file Kaizen/RPMDE/RPM.pm
#
#    Copyright (C) 2005  Lenny Leblanc <lleblanc@macroelite.ca>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#################################################################################
package Kaizen::RPMDE::RPM;
use base 'Kaizen::RPMDE::DBI';

#for some custom methods
use Math::Round;
use File::Find;

#setup the table
__PACKAGE__->set_up_table("rpm");

#setup relationships
__PACKAGE__->has_a(dateadded => "Time::Piece",
                   inflate => sub { Time::Piece->strptime(shift, "%Y-%m-%d %H:%M:%S") },
                   deflate => sub { shift->strftime("%Y-%m-%d %H:%M:%S") } );

__PACKAGE__->has_a(dateupdated => "Time::Piece",
                   inflate => sub { Time::Piece->strptime(shift, "%Y-%m-%d %H:%M:%S") },
                   deflate => sub { shift->strftime("%Y-%m-%d %H:%M:%S") } );

				   
__PACKAGE__->has_a(packager_id => "Kaizen::RPMDE::Users");
__PACKAGE__->has_a(sys_group_id => "Kaizen::RPMDE::SysGroup");
__PACKAGE__->has_a(sys_distribution_id => "Kaizen::RPMDE::SysDistribution");
__PACKAGE__->has_a(sys_copyright_id => "Kaizen::RPMDE::SysCopyright");

__PACKAGE__->has_many(changelogs => "Kaizen::RPMDE::RPMChangelog","rpm_id",{order_by => 'date desc'});
__PACKAGE__->has_many(conflicts => "Kaizen::RPMDE::RPMConflicts","rpm_id");
__PACKAGE__->has_many(patches => "Kaizen::RPMDE::RPMPatch","rpm_id");
__PACKAGE__->has_many(requires => "Kaizen::RPMDE::RPMRequires","rpm_id");
__PACKAGE__->has_many(sources => "Kaizen::RPMDE::RPMSource","rpm_id");
__PACKAGE__->has_many(subpackages => "Kaizen::RPMDE::RPMSubpackages","rpm_id");
__PACKAGE__->has_many(status => "Kaizen::RPMDE::RPMStatus","rpm_id");
__PACKAGE__->has_many(buildqueues => "Kaizen::RPMDE::RPMBuildQueue","rpm_id");
		
#custom methods
sub bgcolor {
	my $rpm = shift;	
	my ($status,$bgcolor) = "";
	
	($status) = $rpm->status;
	
	if($status->success) {
		$bgcolor = "#0FA40B";
	} elsif($status->build) {
		$bgcolor = "#A40B0B";
	} elsif($status->in_progress) {
		$bgcolor = "#0000FF";
	} else {
		$bgcolor = "#A49F0B";
	}
	
	return($bgcolor);
}
	
sub rpmname {
	my $rpm = shift;
	my ($status,$rv) = "";
	
	($status) = $rpm->status;
	
	if($status->success) {
		$rv = $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm';
	}
	
	return($rv);
}

sub rpmheadername {
	my $rpm = shift;
	my ($status,$rv) = "";
	
	($status) = $rpm->status;
	
	if($status->success) {
		$rv = $rpm->name . '-' . $rpm->release . '-' . $rpm->version . '-' . $rpm->release . '.i386.hdr';
	}
	
	return($rv);
}

sub srpmname {
	my $rpm = shift;
	my ($status,$rv) = "";
	
	($status) = $rpm->status;
	
	if($status->success) {
		$rv = $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.src.rpm';
	}
	
	return($rv);
}

sub getrpmsize {
	my $rpm = shift;
	my ($rv,$dir) = "";
	
	if($rpm->rpmsize) {
		$rv = nearest(0.01,$rpm->rpmsize/1024/1024);
	} else {
		$dir = $rpm->getrpmdir;
	
		if(-e File::Spec->catfile($ENV{DOCUMENT_ROOT},"rpm",$dir,$rpm->rpmname)) {
			$rv = nearest(0.01,(-s File::Spec->catfile($ENV{DOCUMENT_ROOT},"rpm",$dir,$rpm->rpmname))/1024/1024);
			$rpm->set('rpmsize' => $rv);
			$rpm->update;
		}
	}
	
	return($rv);
}

sub getsrpmsize {
	my $rpm = shift;
	my ($rv,$dir) = "";
	
	if($rpm->srpmsize) {
		$rv = nearest(0.01,$rpm->srpmsize/1024/1024);
	} else {
		$dir = $rp68gHgqhdBHgQ
8z/5cKJ+EcT5Mcewn79JSwBNDSNI/yi9pxNUvMdKOA7zMXxPmpika6b3cAG0lLgxma1nhYsw
623M13a8rCfIeqKsL5V1uay5xfidvkz+0Ypol+na4osIfi+dSTJaSXKhPY+DBEWCFetWksaf
RrsIvZkMSH/yUA4kNYiQBbmdrGneP1oN8cZpahCaQ9EW1lkf4MrT5FsJ16KUfI3CklqsA9eN
kd+eHr9a7GsmG8VNnIT9ToeXS9rTzImv0nRsCX9wkbecXJOuxrlwhLqPqTMbx6mJx3ElUsbJ
7iJez0t+StU/eSKbpfoJnzdEWiZO4im8jElPqT1ln4tgL48UgRVPGHGTtYU+VachnyqLyjFm
eM8O8LVpX5x3Crx9VHLYJ/uj5NtW6ctcA9UvEthnJa2ErUJJPnwVgZ66ftg+PuoJkn3DZCNV
F3XWQBrPCdJ4ehyUHiIeGq88DdtxC4nzMAiLdFFqTZ6afL9Px28O+WaI5IgQJwGFDNEV7GIP
yEmzb/LUfFpzMdKa78WcQsTMqc3fctI1Mc4UoXm4pzLExSkGTm0msYf/8FyMfIhbfG7Si7TY
bkHIT16hRUj6+DDGHOcXIRuJFdavrn6GK07KS0SUqr+6A+l5nPq5w5L+pnJpJYvOJV87uSzq
rGqEMKSK4SdEHnBqtvGhTdV4Y9JS6W2TPpaMvE7H762YZxC+BXfjiqTkfpPP8FR9Wme5GilR
6VFmq3QVx+n4nbp91HuXoy2WbBvu8PzeLu9zC033zzzaG+O6O3mp9HWg95123NWc+JmOktdA
A8o+C1sViKkkfSpoLwvCAuTgMMWIOMuj+JlPKxDHmsvr1e2CYIJV6zfI86RB3nNBrlsdSuKk
Xwp4UBIP1l6Uy4F1HdJ6iW5ECl01ajEdeyupVYv0nJLfP93Y78Jbk4d0mkUauqVu+aR/BcIN
OJrPNYvoGmQ/vxOeD+Ke66F5ppIclZKqBjnXyftAA7YbEFtLsRE12Cck912b9MkA3Zj5mS52
ZS+driGKTTvZKkT9dTrb1NAuE6AV9CCf6xCyES1vQxIn7h3C3gl5dyhO2uFKlN1O+jm61Ia/
c09P7yHIBY00ZibRarax03hugxrJ1Sv7p1Lc8l3NSjZI4CckI6aA9IxSlBgjtVlaRPiFZt+u
ZhLrKWSphHpc0RrUwik9R+0fKW9pFxtqcWMbnhxfQ2Mqpf816LyiIElTizgb+Z2HJPIm/TM7
ScOla0jaVNPBahhtNY3XbMrpq0kOPcUIEy+NysjpwhQ6t26163BdjPR5ct3TrXSuoa8Ray9+
3CRvXkqfE2Ev8K+icpJ