#################################################################################
#    Class::DBI file Kaizen/RPMDE/RPMBuildQueue.pm
#
#    Copyright (C) 2005  Lenny Leblanc <lleblanc@macroelite.ca>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#################################################################################
package Kaizen::RPMDE::RPMBuildQueue;
use base 'Kaizen::RPMDE::DBI';

use Date::Calc qw(Delta_DHMS);

#setup the table
__PACKAGE__->set_up_table("rpm_buildqueue");

#setup relationships
__PACKAGE__->has_a(start_time => "Time::Piece",
                   inflate => sub { Time::Piece->strptime(shift, "%Y-%m-%d %H:%M:%S") },
                   deflate => sub { shift->strftime("%Y-%m-%d %H:%M:%S") } );

__PACKAGE__->has_a(end_time => "Time::Piece",
                   inflate => sub { Time::Piece->strptime(shift, "%Y-%m-%d %H:%M:%S") },
                   deflate => sub { shift->strftime("%Y-%m-%d %H:%M:%S") } );
				   
__PACKAGE__->has_a(user_id => "Kaizen::RPMDE::Users");
__PACKAGE__->has_a(rpm_id => "Kaizen::RPMDE::RPM");
__PACKAGE__->has_a(build_type => "Kaizen::RPMDE::SysBuilds");


#custom methods
sub get_ttb {
	my $buildqueue = shift;
	my $rv = "";
	
	my ($days, $hours, $minutes, $seconds) = 
			Delta_DHMS( 
				$buildqueue->start_time->year,
				$buildqueue->start_time->mon,
				$buildqueue->start_time->mday,
				$buildqueue->start_time->hour,
				$buildqueue->start_time->min,
				$buildqueue->start_time->sec,
				$buildqueue->end_time->year,
				$buildqueue->end_time->mon,
				$buildqueue->end_time->mday,
				$buildqueue->end_time->hour,
				$buildqueue->end_time->min,
				$buildqueue->end_time->sec
				);
	
	
	if($days eq 1) {$rv .= "1 Day, ";} elsif($days) {$rv .= "$days Days, ";}
	if($hours eq 1) {$rv .= "1 Hour, ";} elsif($hours) {$rv .= "$hours Hours, ";}
	if($minutes eq 1) {$rv .= "1 Minute, ";} elsif($minutes) {$rv .= "$minutes Minutes, ";}
	if($seconds eq 1) {$rv .= "1 Second";} elsif($seconds) {$rv .= "$seconds Seconds";}

	return($rv);
}	
	
1;