#################################################################################
#    Kaizen RPMDE (RPM Development Environment)
#
#    Copyright (C) 2005  Lenny Leblanc <lleblanc@macroelite.ca>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#################################################################################
package Kaizen::RPMDE;

#####################################################
# CONFIG FILE LOCATION
#####################################################
my $config_file = "/etc/rpmde.conf";



#****************************************************
# You do not need to edit anything beyond this point
#****************************************************


##########################################
# A few things that are necessary
##########################################
use strict;
use warnings;

use Apache::RequestRec();
use Apache::RequestIO();
use Apache::Const -compile => qw(OK);
use APR::Const -compile => ':common';

use HTML::Template;
use CGI;
use Mail::Sender;
use CGI::Cookie;
use Log::LogLite;
use Date::Calc qw(:all);
use Time::Piece;
use Math::Round;
use File::Find;
use Net::FTP;
use Digest::MD5;
use Parse::PlainConfig;

# class::dbi modules
use Kaizen::RPMDE::DBI;
use Kaizen::RPMDE::Installations;
use Kaizen::RPMDE::InstallationsMetaGroups;
use Kaizen::RPMDE::InstallationsMetaGroupsLink;
use Kaizen::RPMDE::PendingChanges;
use Kaizen::RPMDE::RPM;
use Kaizen::RPMDE::RPMBuildQueue;
use Kaizen::RPMDE::RPMChangelog;
use Kaizen::RPMDE::RPMConflicts;
use Kaizen::RPMDE::RPMPatch;
use Kaizen::RPMDE::RPMRequires;
use Kaizen::RPMDE::RPMSource;
use Kaizen::RPMDE::RPMStatus;
use Kaizen::RPMDE::RPMSubpackages;
use Kaizen::RPMDE::SysBuilds;
use Kaizen::RPMDE::SysCopyright;
use Kaizen::RPMDE::SysDistribution;
use Kaizen::RPMDE::SysFTPServers;
use Kaizen::RPMDE::SysGroup;
use Kaizen::RPMDE::SysMenu;
use Kaizen::RPMDE::SysSecurity;
use Kaizen::RPMDE::Users;

# comment out the below line when sending to live environment when we're past alpha
use CGI::Carp qw(fatalsToBrowser);

#used for formatting of spec file
my @RPM_NEWLINE_TAGS =	('Name:',
						'Source0:', 'Patch0:',
						'%description', '%prep',
						'%build', '%install',
						'%clean', '%pre',
						'%post', '%preun',
						'%postun', '%verifyscript',
						'%files', '%changelog',
						'%package',);

########################################
# Setup the persistent information
########################################
my %sub_override;			#subroutine overrides
my $conf = "";				#used by Parse::PlainConfig
my $log = "";				#Log::LogLite object
my $q = "";					#CGI object
my $current_user = ""; 		#id of the current user
my $path = "";				#current path
my $path_t = "";			#translated path
my $r = "";					#mod_perl apache object

BEGIN {

	
	%sub_override = (
		''				=> \&login_page,
		'/'				=> \&login_page,
	);
}

########################################
# Handler routine
########################################
sub handler {
	my ($return,$sub,$c) = "";
    
	$r = shift;
	
	# load the configuration file
	$conf = Parse::PlainConfig->new('DELIM' => '=', 'FILE' => $config_file, 'PURGE' => 1);
		
	# create new Log::LogLite object
	$log = new Log::LogLite($conf->get('log_file'), $conf->get('log_level')) or die($conf->get('log_file') . " couldn't be opened");

	# create the CGI object
	$q = CGI->new($r);

	# make some necessary adjustments to the path
	$path = $r->uri;
	$path_t = $path;
	$path_t =~ s/[\/-]/_/g;
	$path_t =~ s/^_//;

	# validate the user
	if(!&check_cookie && $path ne "/error" && $path ne "/report-error") {
		$path_t = "login";
	} else {
		#check for proper access permissions
		my $sys_security = Kaizen::RPMDE::SysSecurity->retrieve(path=>$path);
		$log->write('checking ' . $current_user->userlevel_id . ' vs ' . $sys_security->min_user_level);
		if($current_user->userlevel_id > $sys_security->min_user_level) {
			$path_t = "error";
			$q->param('message',$conf->get('disabled_message'));
			$q->param('suppress_error',1);
		}
	}
	
	# figure out where we are sending control
	$sub = $main::Kaizen::RPMDE::{$path_t . '_page'};
	if(!defined $sub) {
		if(defined $sub_override{$path}) {
			$sub = $sub_override{$path};
		} else {
			$sub = \&error_page;
			$q->param('message','Could not parse the path provided: ' . $path);
		}
	}

	# pass control
	($return,$c) = &{$sub};

	#print the results
	if($return =~ /^REDIRECT:/) { #we have a redirect
		$return =~ s/^REDIRECT://;
		if(defined $c) {
			$log->write("cookie added to header for redirect: $c",5);
			$r->err_headers_out->add("Set-Cookie"=>$c);
		}
		$r->headers_out->set(Location=>$return);

		return Apache::REDIRECT;
	} elsif($return =~ /^IGNORE/) {
		#do nothing
	} else {
		$r->content_type('text/html');
		if(defined $c) {
			$log->write("cookie added to header for normal output: $c",5);
			$r->err_headers_out->add("Set-Cookie"=>$c);
		}
		$r->print($return);

		#return ok
		return Apache::OK;
	}

}

########################################
# Cleanup when our server is shutdown
########################################
END {
	
}

########################################
# Subroutines for the public pages
########################################
########################################
sub admin_page {
	my $admin_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "admin.tpl");
	
	$admin_tpl->param(
				title => "Kaizen::RPMDE::Admin",
				menubar => &get_menubar,
				);
	return($admin_tpl->output);
}

########################################
########################################
sub admin_users_page {
	my $admin_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "admin_users.tpl");
	
	$admin_tpl->param(
				menubar => &get_menubar,
				title => "Kaizen::RPMDE::Admin::Users",
				users => [ map {
							'id' => $_->id,
							'name' => $_->name,
							'email' => $_->email,
							'username' => $_->username,
							'user_level' => $_->userlevel_id->name,
							'chroot_dir' => $_->chroot_dir,
						}, sort {$a->name cmp $b->name}
						map {$_} Kaizen::RPMDE::Users->retrieve_all ],
				);
	return($admin_tpl->output);
}

########################################
########################################
sub admin_users_add_page {
	my $admin_users_add_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "admin_users_add.tpl");
	
	$admin_users_add_tpl->param(
					user => [{ user_levels => [ map {
								 		'id' => $_->id,
										'name' => $_->name,
										}, sort{$a->id cmp $b->id}
										map {$_} Kaizen::RPMDE::UsersLevel->retrieve_all ],
							}],
					);
	return($admin_users_add_tpl->output);
}

########################################
########################################
sub admin_users_delete_page {
	my $user = Kaizen::RPMDE::Users->retrieve($q->param('id'));
	$user->delete;
	
	return("REDIRECT:/admin/users");
}

########################################
########################################
sub admin_users_edit_page {
	my $admin_users_edit_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "admin_users_add.tpl");
    
	my $user = Kaizen::RPMDE::Users->retrieve($q->param('id'));
	
	$admin_users_edit_tpl->param(
					user => [ map { 
								edit => '1',
								id => $_->id,
					            name => $_->name,
								email => $_->email,
								username => $_->username,
								chroot_dir => $_->chroot_dir,
								user_levels => [ map {
										'id' => $_->id,
										'name' => $_->name,
										'selected' => ($_->id eq $user->userlevel_id) ? 'selected' : '',
										}, sort{$a->id cmp $b->id}
										map {$_} Kaizen::RPMDE::UsersLevel->retrieve_all ],
							}, $user],
					);
	return($admin_users_edit_tpl->output);
}

########################################
########################################
sub admin_users_save_page {
	my $popup_close_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "popup_close.tpl");
	my $error_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "error.tpl");	
	
	my %cgi_var = ();
	my ($error,$user) = "";
	
	#setup a hash for params
	$cgi_var{$_} = $q->param($_) foreach $q->param();

	$user = Kaizen::RPMDE::Users->retrieve($cgi_var{'id'});
	
	#double-check the passwords (initial test in javascript)
	if($cgi_var{'password1'} ne $cgi_var{'password2'}) {
		$error = 'Passwords do not match.';
	} elsif(!$cgi_var{'password1'} && !$user) {
		$error = 'Password was left blank.';
	}
	
	if($error) {
		$error_tpl->param('message' => $error);
		return($error_tpl->output);
	} else {
		if($cgi_var{'password1'}) {
			$cgi_var{'password'} = $cgi_var{'password1'};
		} 
		
		delete($cgi_var{'id'});
		delete($cgi_var{'password1'});
		delete($cgi_var{'password2'});		
		
		if($user) {
			#we're updating
			$user->set(%cgi_var);
			$user->update;
			
			return($popup_close_tpl->output);
		} else {
			my $user = Kaizen::RPMDE::Users->create(\%cgi_var);
			if($user) {				
				return($popup_close_tpl->output);
			} else {
				$error_tpl->param('message' => "Error adding user to database.");
				return($error_tpl->output);
			}
		}
	}
}


########################################
########################################
sub buildrpm_page {
	my $buildrpm_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "buildrpm.tpl");
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param("id"));
	my @builds = Kaizen::RPMDE::SysBuilds->retrieve_all_sorted_by("build_order");
	
	my ($rpmfile,$srpmfile,$status) = "";
    ($status) = $rpm->status;
	
	my @completed = map {
					user => $_->user_id->name,
					user_email => $_->user_id->email,
					start_time => $_->start_time->strftime("%Y-%m-%d %H:%M:%S"),
					end_time => $_->end_time->strftime("%Y-%m-%d %H:%M:%S"),
					ttb => $_->get_ttb,
					rpm_id => $rpm->id,
					id => $_->id,
				}, sort {$b->start_time cmp $a->start_time}
				map {$_} Kaizen::RPMDE::RPMBuildQueue->search_where(rpm_id=>$rpm->id,status=>'done');
	
	$buildrpm_tpl->param(
			title => "Kaizen::RPMDE::RPMS::Build",
			menubar => &get_menubar,
			id => $q->param('id'),
			website => $rpm->url,
			rpmname => $rpm->name,
			rpmdir => ($status->success) ? $rpm->getrpmdir : '',
			srpmdir => ($status->success) ? $rpm->getsrpmdir : '',
			rpmfile => ($status->success) ? $rpm->rpmname : '',
			srpmfile => ($status->success) ? $rpm->srpmname : '',
			completed => \@completed,
			noqueue => 1,
			builds =>
				[map {
					'value' => $_->id,
					'option' => $_->description}, @builds],
			);
			
	return($buildrpm_tpl->output);
}

########################################
########################################
sub buildrpm_delete_page {
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('rpm_id'));
	my $buildqueue = Kaizen::RPMDE::RPMBuildQueue->retrieve($q->param('id'));
	
	#delete the actual build log
	$log->write("removing log file /var/www/rms/" . $rpm->getbuildlogdir . $buildqueue->log_file);
	unlink('/var/www/rms/' . $rpm->getbuildlogdir . $buildqueue->log_file);
	
	#delete the build from the database
	$buildqueue->delete;
	
	#redirect back to the rpm build page
	return("REDIRECT:/buildrpm?id=" . $q->param('rpm_id'));
}

########################################
########################################
sub buildrpm_viewlog_page {
	my $buildrpm_viewlog_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "buildrpm_viewlog.tpl");
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('rpm_id'));
	my $buildqueue = Kaizen::RPMDE::RPMBuildQueue->retrieve($q->param('id'));
	
	local $/;
	
	open(FH,$ENV{DOCUMENT_ROOT} . '/' . $rpm->getbuildlogdir . $buildqueue->log_file);
	my $build_log = <FH>;
	close(FH);
	
	$buildrpm_viewlog_tpl->param(
			title => "Kaizen::RPMDE::RPMS::Build::ViewLog",
			menubar => &get_menubar,
			rpmname => $rpm->name,
			id => $rpm->id,
			build_log => $build_log,
			);	
	return($buildrpm_viewlog_tpl->output);
}

########################################
########################################
sub deleterpm_page {
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param("id"));
	
	$rpm->delete;
	
	return("REDIRECT:/managerpm");
}

########################################
########################################
sub deletesubpackage_page {
	my $popup_close_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "popup_close.tpl");
	my $subpackage = Kaizen::RPMDE::RPMSubpackages->retrieve($q->param("id"));
	my $rpm = Kaizen::RPMDE::RPM->retrieve($subpackage->rpm_id);
	my $pending_change = "";
		
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {
		#delete the subpackage
		$subpackage->delete;
	
		#let's update the rpm status
		&update_rpm_status($rpm,'0','0','0');
		return($popup_close_tpl->output);
	
	} elsif($current_user->userlevel_id < '40') {
		#pending change
		$pending_change = Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $subpackage->id,
					'table_name' => 'rpm_subpackage',
					'field_name' => '_DELETE_',
		});
				
		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}

	return($popup_close_tpl->output);	
}

########################################
########################################
sub editrpm_page {
	my $newrpm_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newrpm.tpl");
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
	my ($i,$group_id) = "";
	
	my @rpms = 
			({'edit' => 1, 'id' => $rpm->id,
			'name' => $rpm->name, 'version' => $rpm->version,
			'release' => $rpm->release, 'description' => $rpm->description,
			'summary' => $rpm->summary,
			'copyright_id' => $rpm->sys_copyright_id,
			'distribution_id' => $rpm->sys_distribution_id,
			'group_id' => $rpm->sys_group_id,
			'packager_id' => $rpm->packager_id,
			'icon' => $rpm->icon, 'vendor' => $rpm->vendor,
			'url' => $rpm->url, 'serial' => $rpm->serial,
			'autoreqprov' => $rpm->autoreqprov, 'provides' => $rpm->provides,
			'excludearch' => $rpm->excludearch, 'exclusivearch' => $rpm->exclusivearch,
			'excludeos' => $rpm->excludeos, 'exclusiveos' => $rpm->exclusiveos,
			'prefix' => $rpm->prefix, 'buildroot' => $rpm->buildroot,
			'defines' => $rpm->defines, 'prep' => $rpm->prep, 'build' => $rpm->build,
			'install' => $rpm->install, 'clean' => $rpm->clean,
			'pre' => $rpm->pre, 'post' => $rpm->post,
			'preun' => $rpm->preun, 'postun' => $rpm->postun,
			'files' => $rpm->files, 'verifyscript' => $rpm->verifyscript,
			'changelog' => 
				[map {'date' => ('* ' . $_->date->day . ' ' . $_->date->month . ' '  . $_->date->mday
								. ' ' . $_->date->year . ' ' . $rpm->packager_id->name
								. ' &lt;' . $rpm->packager_id->email . '&gt;'),
					  'description' => sub_newline($_->description),},
					$rpm->changelogs],
			'sources' => 
				[map {'id' => $_->id,
					  'sourcedir' => '/rpm/SOURCES/',
					  'source' => $_->source,
					  'include' => ($_->include) ? '1' : '',
					  'original' => $_->original,
					  'size' => nearest(0.01,(-s File::Spec->catfile($ENV{DOCUMENT_ROOT},"rpm","SOURCES",$_->source))/1024/1024),},
					$rpm->sources],
			'patches' => 
				[map {'id' => $_->id,
					  'patch' => $_->patch,
					  'patchdir' => '/rpm/SOURCES/',
					  'include' => ($_->include) ? '1' : '',
					  'size' => nearest(0.01,(-s File::Spec->catfile($ENV{DOCUMENT_ROOT},"rpm","SOURCES",$_->patch))/1024),},
					$rpm->patches],
			'requires' =>
				[map {'name' => $_->name,
					  'version' => $_->version},
				$rpm->requires],
			'subpackages' =>
				[map {'id' => $_->id,
					  'name' => $_->name,
					  'summary' => $_->summary,
					  'description' => $_->description,
					  'group' => $_->sys_group_id,
					  'pre' => $_->pre,
					  'post' => $_->post,
					  'preun' => $_->preun,
					  'postun' => $_->postun,
					  'verifyscript' => $_->verifyscript,
					  'files' => $_->files,
					  },
				$rpm->subpackages],
			'subpackages_menu' =>
				[map {'name' => $_->name,},
				$rpm->subpackages],
			});
	
	#setup the drop down lists
	$rpms[0]{'copyright'} =
			[map {
				'value' => $_->id, 
				'option' => ($_->name . ' - ' . $_->description),
				'selected' => ($_->id eq $rpms[0]{'copyright_id'}) ? ' selected' : ''},
			sort {$a->name cmp $b->name}
			map {$_} Kaizen::RPMDE::SysCopyright->retrieve_all];
	$rpms[0]{'distribution'} = 					
			[map {
				'value' => $_->id, 
				'option' => $_->name,
				'selected' => ($_->id eq $rpms[0]{'distribution_id'}) ? ' selected' : ''}, 
			sort {$a->name cmp $b->name}
			map {$_} Kaizen::RPMDE::SysDistribution->retrieve_all];
	$rpms[0]{'group'} =
			[map {
				'value' => $_->id, 
				'option' => $_->name,
				'selected' => ($_->id eq $rpms[0]{'group_id'}) ? ' selected' : ''}, 
			sort {$a->name cmp $b->name}
			map {$_} Kaizen::RPMDE::SysGroup->retrieve_all];
	$rpms[0]{'packager'} =
			[map {
				'value' => $_->id, 
				'option' => $_->name,
				'selected' => ($_->id eq $rpms[0]{'packager_id'}) ? ' selected' : ''}, 
			sort {$a->name cmp $b->name}
			map {$_} Kaizen::RPMDE::Users->search_where('userlevel_id' => ['10','20'])];

	#setup the group drop down list for all the subpackages
	for(my $i=0;$i <= $#{$rpms[0]{'subpackages'}};$i++) {
		$group_id = $rpms[0]{'subpackages'}[$i]{'group'};
		$rpms[0]{'subpackages'}[$i]{'group'} =
			[map {
				'value' => $_->id,
				'option' => $_->name,
				'selected' => ($_->id eq $group_id) ? ' selected' : ''},
			sort {$a->name cmp $b->name}
			map {$_} Kaizen::RPMDE::SysGroup->retrieve_all];
	}
	
	#delete our temporary placeholders
	delete($rpms[0]{'copyright_id'});
	delete($rpms[0]{'distribution_id'});
	delete($rpms[0]{'group_id'});
	delete($rpms[0]{'packager_id'});
			
	$newrpm_tpl->param(
			title => "Kaizen::RPMDE::RPMS::Edit",
			menubar => &get_menubar,
			edit => 1,
			id => $q->param('id'),
			website => $rpm->url,
			rpmdata => \@rpms);
    
	return($newrpm_tpl->output);
}


###############################################
###############################################
sub editrpm_editpatch_page {
	my $editrpm_editpatch_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newpatch.tpl");
	my $patch = Kaizen::RPMDE::RPMPatch->retrieve($q->param('patch_id'));
				 
	$editrpm_editpatch_tpl->param(
				id=>$patch->id,
				include => ($patch->include eq '1') ? 'checked' : '',
				edit=>1,
				);
	return($editrpm_editpatch_tpl->output);
}		

###############################################
###############################################
sub editrpm_deletepatch_page {
	my $popup_close_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "popup_close.tpl");
	my $patch = Kaizen::RPMDE::RPMPatch->retrieve($q->param('patch_id'));
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $patch->rpm_id->packager_id)) {
		#delete the patch
		unlink($conf->get('sources_dir') . '/' . $patch->patch);
		$patch->delete;
	
		return($popup_close_tpl->output);
	} elsif($current_user->userlevel_id < '40') {
		#pending change
		Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $patch->id,
					'table_name' => 'rpm_patch',
					'field_name' => '_DELETE_',
		});

		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}
				
}

###############################################
###############################################
sub editrpm_updatepatch_page {
	my $popup_close_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "popup_close.tpl");
	my $include = $q->param("include");
	
	my $patch= Kaizen::RPMDE::RPMPatch->retrieve($q->param('id'));
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $patch->rpm_id->packager_id)) {
		#do the update	
		$patch->set(include => $include);
		
		my $rpm = Kaizen::RPMDE::RPM->retrieve($patch->rpm_id->id);
		&update_rpm_status($rpm,'0','0','0');
		
		$patch->update;
		return($popup_close_tpl->output);
	} elsif($current_user->userlevel_id < '40') {
		#pending change
		Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $patch->id,
					'table_name' => 'rpm_patch',
					'field_name' => 'include',
					'proposed_value' => $include,
		});

		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}
}

###############################################
###############################################
sub editrpm_editsource_page {
	my $editrpm_editsource_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newsource.tpl");
	my $source = Kaizen::RPMDE::RPMSource->retrieve($q->param('source_id'));
				 
	$editrpm_editsource_tpl->param(
				source_id=>$source->id,
				original => $source->original,
				include => ($source->include eq '1') ? 'checked' : '',
				edit=>1,
				);
	return($editrpm_editsource_tpl->output);
}					

###############################################
###############################################
sub editrpm_deletesource_page {
	my $popup_close_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "popup_close.tpl");
	my $source = Kaizen::RPMDE::RPMSource->retrieve($q->param('source_id'));

	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $source->rpm_id->packager_id)) {	
		#delete the source
		unlink($conf->get('sources_dir') . '/' . $source->source);
		$source->delete;
		return($popup_close_tpl->output);
	} elsif($current_user->userlevel_id < '40') {
		#pending deletion
		Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $source->id,
					'table_name' => 'rpm_source',
					'field_name' => '_DELETE_',
		});
		
		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}
}

###############################################
###############################################
sub editrpm_updatesource_page {
	my $popup_close_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "popup_close.tpl");
	my $source_id = $q->param("source_id");
	my $original = $q->param("original");
	my $include = $q->param("include");
	
	my $source = Kaizen::RPMDE::RPMSource->retrieve($source_id);
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $source->rpm_id->packager_id)) {
		#make the change
		$source->set(original => $original, include => $include);
	
		my $rpm = Kaizen::RPMDE::RPM->retrieve($source->rpm_id->id);
		&update_rpm_status($rpm,'0','0','0');
		
		$source->update;
	
		return($popup_close_tpl->output);
	} elsif($current_user->userlevel_id < '40') {
		#pending change (only make it if it's changed)
		if($source->original ne $original) {
			Kaizen::RPMDE::PendingChanges->create({
						'user_id' => $current_user->id,
						'primary_id' => $source->id,
						'table_name' => 'rpm_source',
						'field_name' => 'original',
						'proposed_value' => $original,
			});
		}
		
		if($source->include ne $include) {		
			Kaizen::RPMDE::PendingChanges->create({
						'user_id' => $current_user->id,
						'primary_id' => $source->id,
						'table_name' => 'rpm_source',
						'field_name' => 'include',
						'proposed_value' => $include,
			});
		}
		
		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}		
}

###############################################
###############################################
sub error_page {
	my $message = $q->param("message");
	my $suppress_error = $q->param("suppress_error");
	
	my $error_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "error.tpl");

	$error_tpl->param(
			message=>$message,
			suppress_error=>$suppress_error
	);
	return($error_tpl->output);
}

########################################
########################################
sub executerpm_page {
	my $buildrpm_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "buildrpm.tpl");
	my @subpackages = ();
	my $queued = 0;
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));    
	my $build = Kaizen::RPMDE::SysBuilds->retrieve($q->param('build_type'));
    my $pending_change = "";
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {
		#check if we already have a build sceduled for this rpm
		my $buildqueue = Kaizen::RPMDE::RPMBuildQueue->search_where(rpm_id => $rpm->id,status => { '!=', 'done'});
		
		if($buildqueue) {
			$queued = 0;
		} else {
			#schedule the build
			my $build = $rpm->add_to_buildqueues({
									'rpm_id' => $q->param('id'),
									'build_type' => $q->param('build_type'),
									'user_id' => $current_user->id
									});	
			$queued = 1;
		}
		
		$buildrpm_tpl->param(
				title => "Kaizen::RPMDE::RPMS::Build",
				menubar => &get_menubar,
				id => $q->param('id'),
				rpmname => $rpm->name,
				website => $rpm->url,
				queued => $queued,
				);
		
		return($buildrpm_tpl->output);
	} elsif($current_user->userlevel_id < '40') {
		#pending build
		$pending_change = Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $rpm->id,
					'table_name' => 'rpm_buildqueue',
					'field_name' => '_BUILD_',
					'proposed_value' => $build->id,
		});

		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}
}
	

########################################
########################################
sub generatespec_page {
	my $spec_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "spec.tpl");
	my ($rpm,$status,$spec_file) = "";
	    
	$rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
    ($status) = $rpm->status;
	
	if(! $status->spec) {
		$log->write("ouputting the spec file to disk");
		
		&output_spec($rpm);
		
		#since we wrote the spec file to disk, let's update the spec file status
		#so we don't rewrite this everytime (unless there is a change of course)
		&update_rpm_status($rpm,'1','0','0');
	}
	
	#this will allow us to read the entire file into a single string
	local $/=undef;
	
	#open the file for input
	open(SPEC,"<" . $conf->get('specs_dir') . "/" . $rpm->name . "-" . $rpm->version . "-" . $rpm->release . ".spec");
	
	#read the file into a string
	binmode SPEC;
	$spec_file = <SPEC>;
	
	#close the file
	close(SPEC);	
	
	$spec_tpl->param(
			title => "Kaizen::RPMDE::RPMS:Spec",
			menubar => &get_menubar,
			rpmname => $rpm->name . '-' . $rpm->version,
			id => $q->param('id'),
			website => $rpm->url,
			spec_file => $spec_file);
    
	return($spec_tpl->output);
}

########################################
########################################
sub home_page {
	my $home_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "home.tpl");
	
	$home_tpl->param(menubar => &get_menubar,);
			
	return($home_tpl->output);
}

########################################
########################################
sub installation_page {
	my $installation_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation.tpl");
	
	my @installations = Kaizen::RPMDE::Installations->retrieve_all;
	                               
	$installation_tpl->param(
			menubar => &get_menubar,
			installations => 
					[map {
						'id' => $_->id,
						'name' => $_->name,
						'description' => $_->description,
						'dateupdated' => $_->dateupdated->ymd,},
					sort {$a->name cmp $b->name}
					map {$_} @installations ],
			);
	return($installation_tpl->output);
}

########################################
########################################
sub installation_edit_page {
	my $installation_edit_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_new.tpl");
	my $id = $q->param("id");
	
	my $installation = Kaizen::RPMDE::Installations->retrieve($id);
	
	$installation_edit_tpl->param(
			menubar => &get_menubar,
			edit => '1',
			id => $id,
			name => $installation->name,
			description => $installation->description,
			);
	return($installation_edit_tpl->output);
}

########################################
########################################
sub installation_delete_page {
	my $id = $q->param("id");
	
	#get the installation object and delete
	my $installation = Kaizen::RPMDE::Installations->retrieve($id);
	$installation->delete;
	
	return("REDIRECT:/installation");
}

########################################
########################################
sub installation_new_page {
	my $installation_new_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_new.tpl");
	
	$installation_new_tpl->param(menubar => &get_menubar);
	return($installation_new_tpl->output);
}

########################################
########################################
sub installation_store_page {
	my $name = $q->param("name");
	my $description = $q->param("description");
	
	my $installation = Kaizen::RPMDE::Installations->create({
														name=>$name,
														description=>$description,
														dateadded=>Time::Piece->new,
														dateupdated=>Time::Piece->new,
														});
	
	return("REDIRECT:/installation");
}

########################################
########################################
sub installation_update_page {
	my $id = $q->param("id");
	my $name = $q->param("name");
	my $description = $q->param("description");
	
	my $installation = Kaizen::RPMDE::Installations->retrieve($id);
	
	$installation->set(name=>$name,
					   description=>$description,
					   dateupdated=>Time::Piece->new);
	$installation->update;
		
	return("REDIRECT:/installation");
}

########################################
########################################
sub installation_buildiso_page {
	my $installation_buildiso_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_buildiso.tpl");
	my $id = $q->param('id');

	$installation_buildiso_tpl->param(menubar => &get_menubar,
									  id => $id);
	return($installation_buildiso_tpl->output);
}

########################################
########################################
sub installation_buildiso_doit_page {
	my $installation_buildiso_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_buildiso.tpl");
	my ($command,$output,$rpm) = "";
	my @subpackages = ();
		
	my $id = $q->param('id');
    $output = "<b>Copying RPMs to " . $conf->get('install_base') . "</b><br><BR><I>";
	
	#for now let's get all the latest rpms to INSTALL_BASE
	my @rpms = Kaizen::RPMDE::RPM->retrieve_all();
	
	foreach $rpm (@rpms) {
		#copy the rpm
		$command = "sudo cp /var/www/rms/rpm/" . $rpm->getrpmdir . $rpm->rpmname . " " . $conf->get('install_base') . "/RPM";
		$output .= qx($command);
		$output .= "$command<br>";
		
		#copy the srpm
		$command = "sudo cp /var/www/rms/rpm/" . $rpm->getsrpmdir . $rpm->srpmname . " " . $conf->get('install_base') . "/SRPM";
		$output .= qx($command);
		$output .= "$command<br>";
		
		#copy all of the subpackage rpm files
		@subpackages = Kaizen::RPMDE::RPMSubpackages->search(rpm_id=>$rpm->id);
			
		foreach(@subpackages) {
			$command = "sudo cp /var/www/rms/rpm/" . $rpm->getrpmdir . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm ' . $conf->get('install_base') . '/RPM';
			$output .= qx($command);
			$output .= "$command<br>";
		}					
	}
	
	$output .= "</I><br><BR><b>All Done!</b>";		
			
	$installation_buildiso_tpl->param(menubar => &get_menubar,
									  id => $id,
									  output => $output);
	return($installation_buildiso_tpl->output);
}

########################################
########################################
sub installation_comps_page {
	my $output_comps_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "output_comps.tpl");
	my $id = $q->param("id");
    my (@hierarchy,@group_list,@rpms) = ();
	my ($main,$pmain,$sub) = "";
	my ($count,$count2,$flag) = 0;
		
	my @groups = Kaizen::RPMDE::SysGroup->retrieve_all();
	
	#setup the hierarchy array
	foreach(@groups) {
		($main,$sub) = split(/\//,$_->name);

		if($main eq $pmain) {
			$hierarchy[$count]{'groups'}[$count2]{'group'} = $_->textid;
			$count2++;
		} else {
			$count2=0;
			if($flag) { $count++; $flag=0; }
			
			$hierarchy[$count]{'name'} = $main;
			$hierarchy[$count]{'groups'}[$count2]{'group'} = $_->textid;
			$count2++;
			$pmain = $main;
			$flag=1;
		}			
	}
	
	#setup the groups array
	$count=0;
	foreach(@groups) {
		($main,$sub) = split(/\//,$_->name);
		
		$group_list[$count]{'id'} = $_->textid;
		$group_list[$count]{'name'} = $sub;
		$group_list[$count]{'def_value'} = $_->def_value;
		$group_list[$count]{'description'} = $_->description;
		$group_list[$count]{'uservisible'} = $_->uservisible;
		
		@rpms = $_->rpms;
		
		$count2=0;
		foreach(@rpms) {
			$group_list[$count]{'packagelist'}[$count2]{'type'} = $_->type;
			$group_list[$count]{'packagelist'}[$count2]{'name'} = $_->name;
			$count2++;
		}
		
		$count++;
	}
	
	$output_comps_tpl->param(
			#menubar => &get_menubar,
			#id => $id,
			group_list => \@group_list,
			group_hierarchy => \@hierarchy
			);
	
	return($output_comps_tpl->output);
}

########################################
########################################
sub installation_grouphierarchy_page {
	my $installation_grouphierarchy_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_grouphierarchy.tpl");
	my $id = $q->param("id");
    my @hierarchy = ();
	my ($main,$pmain,$sub) = "";
	my ($count,$count2,$flag) = 0;
		
	my @groups = Kaizen::RPMDE::SysGroup->retrieve_all();
	
	foreach(@groups) {
		($main,$sub) = split(/\//,$_->name);
		
		if($main eq $pmain) {
			$hierarchy[$count]{'groups'}[$count2]{'group'} = $sub;
			$count2++;
		} else {
			$count2=0;
			if($flag) { $count++; $flag=0; }
			
			$hierarchy[$count]{'name'} = $main;
			$hierarchy[$count]{'groups'}[$count2]{'group'} = $sub;
			$count2++;
			$pmain = $main;
			$flag=1;
		}			
	}
	
	$installation_grouphierarchy_tpl->param(
			menubar => &get_menubar,
			id => $id,
			group_hierarchy => \@hierarchy
			);
	
	return($installation_grouphierarchy_tpl->output);
}									

########################################
########################################
sub installation_metagroups_page {
	my $installation_metagroups_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_metagroups.tpl");
	my $id = $q->param("id");	
		
	$installation_metagroups_tpl->param(
			menubar => &get_menubar,
			id => $id,
			meta_groups => [map {
								'id' => $id,
								'meta_group_id' => $_->id,
								'name' => $_->name,
								'uservisible' => $_->uservisible,
								'groups' => [map {
												'name' => $_->sys_group_id->name,},
												sort {$a->sys_group_id->name cmp $b->sys_group_id->name}
												map {$_} $_->links],},
					 		sort {$a->name cmp $b->name}
					 		map {$_} Kaizen::RPMDE::InstallationsMetaGroups->search(installation_id => $id)],
										);
	return($installation_metagroups_tpl->output);
}

########################################
########################################
sub installation_metagroups_edit_page {
	my $installation_metagroups_edit_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_metagroups_new.tpl");
	my $id = $q->param("id");
	my $meta_group_id = $q->param("meta_group_id");	
		
	#setup the template
	$installation_metagroups_edit_tpl->param(
			menubar => &get_menubar,
			id => $id,
			meta_group_id => $meta_group_id,
			edit => '1',
			meta_groups => [ map {
								'name' => $_->name,
								'uservisible' => ($_->uservisible eq 'true') ? 'checked' : '',
								'groups' => [ map {
												'id' => $_->id,
												'name' => $_->name,
												'selected' => $_->meta_group_check($meta_group_id)},
											sort {$a->name cmp $b->name}
											map {$_} Kaizen::RPMDE::SysGroup->retrieve_all ],},
							sort {$a->name cmp $b->name}
							map {$_} Kaizen::RPMDE::InstallationsMetaGroups->retrieve($meta_group_id)],
			);
		
	return($installation_metagroups_edit_tpl->output);
}

########################################
########################################
sub installation_metagroups_delete_page {
	my $id = $q->param("id");
	my $meta_group_id = $q->param("meta_group_id");	
		
	#get the meta group object
	my $meta_group = Kaizen::RPMDE::InstallationsMetaGroups->retrieve($meta_group_id);
	
	#delete the meta group
	$meta_group->delete;
	
	return("REDIRECT:/installation/metagroups?id=$id");
}

########################################
########################################
sub installation_metagroups_new_page {
	my $installation_metagroups_new_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "installation_metagroups_new.tpl");
	my $id = $q->param("id");	
	
	$installation_metagroups_new_tpl->param(
			menubar => &get_menubar,
			id => $id,
			meta_groups => [ {'groups' => [ map {
											'id' => $_->id,
											'name' => $_->name,},
											sort {$a->name cmp $b->name}
											map {$_} Kaizen::RPMDE::SysGroup->retrieve_all],
						   } ]
			);
	return($installation_metagroups_new_tpl->output);
}

########################################
########################################
sub installation_metagroups_store_page {
	my $id = $q->param("id");
	my $name = $q->param("name");
	my $uservisible = $q->param("uservisible");
	my @groups = $q->param("groups");

	if(!defined $uservisible || $uservisible eq "") { $uservisible='false';}
		
	#get the installation object 
	my $installation = Kaizen::RPMDE::Installations->retrieve($id);
	
	#create the meta group
	my $meta_group = $installation->add_to_metagroups({name=>$name,uservisible=>$uservisible});
	
	#store the groups for the meta group
	foreach(@groups) {
		$meta_group->add_to_links({sys_group_id=>$_});
	}	

	return("REDIRECT:/installation/metagroups?id=$id");
}

########################################
########################################
sub installation_metagroups_update_page {
	my $id = $q->param("id");
	my $meta_group_id = $q->param("meta_group_id");
	my $name = $q->param("name");
	my $uservisible = $q->param("uservisible");
	my @groups = $q->param("groups");

	if(!defined $uservisible || $uservisible eq "") { $uservisible='false';}
		
	#get the meta group
	my $meta_group = Kaizen::RPMDE::InstallationsMetaGroups->retrieve($meta_group_id);

	#update the meta group
	$meta_group->set(name=>$name,uservisible=>$uservisible);
	
	#delete the group links
	my @links = $meta_group->links;
	foreach(@links) {
		$_->delete;
	}
		
	#store the groups for the meta group
	foreach(@groups) {
		$meta_group->add_to_links({sys_group_id=>$_});
	}	

	#update the meta group
	$meta_group->update;
	
	return("REDIRECT:/installation/metagroups?id=$id");
}

########################################
########################################
sub login_page {
	my $c = "";

	my $login_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "login.tpl");
	my $username = $q->param("username");
	my $password = $q->param("password");

	if(defined $username) {
		$c = &get_user_cookie($username,$password);
		if(defined $c) {
			return("REDIRECT:/search",$c);
		} else {
			$login_tpl->param(invalid=>"true");
		}
	}

	$login_tpl->param(title=>"Kaizen::RPMDE::Login");
	return($login_tpl->output);
}

########################################
########################################
sub logout_page {
	my $c = "";	
	
	if(defined $current_user) {
		$c = $q->cookie(
				-name			=>'credentials',
				-value			=>"username:" . $current_user->username . ":password:" . $current_user->password,
			    -expires		=>'-1d',
			    -domain			=>$conf->get('domain'),
				-path			=>'/');
	}
	
	return("REDIRECT:/",$c);
}

########################################
########################################
sub managerpm_page {
	my $managerpm_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "managerpm.tpl");
	
	my @rpms = Kaizen::RPMDE::RPM->retrieve_all;
	
	my ($total_rpm_size,$total_srpm_size) = 0;
	foreach(@rpms) {
		$total_rpm_size += $_->rpmsize;
		$total_srpm_size += $_->srpmsize;
	}
		
	$managerpm_tpl->param(
			title => "Kaizen::RPMDE::RPMS::Manage",
			menubar => &get_menubar,
			total_rpms => Kaizen::RPMDE::RPM->count_search,
			total_size => nearest(0.01,$total_rpm_size/1024/1024),
			total_ssize => nearest(0.01,$total_srpm_size/1024/1024),
			rpms => 
					[map {
						'id' => $_->id,
						'name' => $_->name,
						'version' => $_->version,
						'dateupdated' => $_->dateupdated->ymd,
						'packager' => $_->packager_id->name,
						'packager_email' => $_->packager_id->email,
						'website' => $_->url,
						'bgcolor' => $_->bgcolor,
						'rpm' => $_->getrpmdir . $_->rpmname,
						'srpm' => $_->getsrpmdir . $_->srpmname,
						'rpmsize' => $_->getrpmsize,
						'srpmsize' => $_->getsrpmsize,},
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::RPM->retrieve_all]);
    
	return($managerpm_tpl->output);
}

########################################
########################################
sub newchangelog_page {
	my $newchangelog_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newchangelog.tpl");
	
	$newchangelog_tpl->param(id=>$q->param('id'));
	
	return($newchangelog_tpl->output);
}

########################################
########################################
sub newpatch_page {
	my $newpatch_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newpatch.tpl");
	
	$newpatch_tpl->param(id=>$q->param('id'));
	
	return($newpatch_tpl->output);
}

########################################
########################################
sub newrequire_page {
	my $newrequire_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newrequire.tpl");

	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
	
	$newrequire_tpl->param(
		id => $q->param('id'),
		rpms =>
			[map {'value' => $_->id, 'option' => ($_->name . ' - ' . $_->version)},
			sort {$a->name cmp $b->name}
			map {$_} Kaizen::RPMDE::RPM->retrieve_all]);
	
	return($newrequire_tpl->output);
}

########################################
########################################
sub newrpm_page {
	my $newrpm_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newrpm.tpl");
	
	$newrpm_tpl->param(
			title => "Kaizen::RPMDE::RPMS::New",
			menubar => &get_menubar,
			new => '1',
			rpmdata => [{
				release => '1',
				serial => '1',
				exclusivearch => 'i386',
				exclusiveos => 'linux',
				buildroot => '%{_tmppath}/%{name}-root',
				prep => '%setup',
				install => '[ %{buildroot} != "/" ] && rm -rf %{buildroot}',
				clean => '#[ %{buildroot} != "/" ] && rm -rf %{buildroot}',
				copyright =>
					[map {'value' => $_->id, 'option' => ($_->name . ' - ' . $_->description)},
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::SysCopyright->retrieve_all],
				distribution => 
					[map {'value' => $_->id, 'option' => $_->name}, 
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::SysDistribution->retrieve_all],
				group => 
					[map {'value' => $_->id, 'option' => $_->name}, 
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::SysGroup->retrieve_all],
				packager => 
					[map {'value' => $_->id, 'option' => $_->name}, 
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::Users->search_where(userlevel_id => [ '10', '20' ])],
				vendor => "Kaizen"
			}]);
	return($newrpm_tpl->output);
}

########################################
########################################
sub newsource_page {
	my $newsource_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newsource.tpl");
	
	$newsource_tpl->param(id=>$q->param('id'));
	return($newsource_tpl->output);
}

########################################
########################################
sub newsubpackage_page {
	my $newsubpackage_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newsubpackage.tpl");
	
	$newsubpackage_tpl->param(id=>$q->param('id'));
	return($newsubpackage_tpl->output);
}

########################################
########################################
sub rpms_page {
	my $rpms_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "rpms.tpl");

	$rpms_tpl->param(
		title => "Kaizen::RPMDE::RPMS",
		menubar=>&get_menubar
		);	
	return($rpms_tpl->output);
}

########################################
########################################
sub savechangelog_page {
	my $popup_close_tpl = new HTML::Template(filename => $conf->get('template_dir') . "popup_close.tpl");
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
	my $pending_change = "";
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {	
		#add the changelog entry
		my $changelog = $rpm->add_to_changelogs({
				'user_id' => $current_user->id,
				'date' => Time::Piece->new,
				'description' => $q->param('changelog')});
			
		&update_rpm_status($rpm,'0','0','0');		
	
		if($changelog) {
			return($popup_close_tpl->output);	
		} else {
			$q->param('message',$conf->get('dberror_message'));
			return(&error_page);
		}
	} elsif($current_user->userlevel_id < '40') {
		#pending change
		$pending_change = Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $rpm->id,
					'table_name' => 'rpm_changelog',
					'field_name' => '_NEW_',
					'proposed_value' => $q->param('changelog'),
		});
		
		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}
}

########################################
########################################
sub savepatch_page {
	my $popup_close_tpl = new HTML::Template(filename => $conf->get('template_dir') . "popup_close.tpl");
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
	my ($patch,$pending_change) = '';
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {
		#save the patch
		if(&save_file($q->param("patch"))) {
			my $md5 = Digest::MD5->new;
			open(FH,$conf->get('sources_dir') . '/' . $q->param('patch'));
			binmode(FH);
			$md5->addfile(*FH);
			close(FH);
			
			$patch = $rpm->add_to_patches({
				'patch' => $q->param('patch'),
				'include' => ($q->param('include')) ? '1' : '0',
				'size' => (-s $conf->get('sources_dir') . '/' . $q->param('patch')),
				'md5sum' => $md5->hexdigest,
				});
			
			&update_rpm_status($rpm,'0','0','0');
			
			if($patch) {
				return($popup_close_tpl->output);
			} else {
				return('REDIRECT:/home');
			}
		} else {
			return('REDIRECT:/home');
		}
	} elsif($current_user->userlevel_id < '40') {
		#pending change
		if(&save_file($q->param('patch'),1)) {
			$pending_change = Kaizen::RPMDE::PendingChanges->create({
						'user_id' => $current_user->id,
						'primary_id' => $rpm->id,
						'table_name' => 'rpm_patch',
						'field_name' => '_NEW_',
						'proposed_value' => $q->param('patch'),
			});
			
			Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $pending_change->id,
					'table_name' => 'rpm_patch_new',
					'field_name' => 'include',
					'proposed_value' => $q->param('include'),});
		}		
					
		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}
}

########################################
########################################
sub saverequire_page {
	my $popup_close_tpl = new HTML::Template(filename => $conf->get('template_dir') . "popup_close.tpl");
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
	my $req = Kaizen::RPMDE::RPM->retrieve($q->param('req'));
    my $pending_change = "";
		
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {
		#save the requirement
		my $require = $rpm->add_to_requires({
				'rpm_id' => $rpm->id,
				'name' => $req->name,
				'version' => $req->version});
		
		&update_rpm_status($rpm,'0','0','0');		

		if($require) {
			return($popup_close_tpl->output);	
		} else {
			$q->param('message',$conf->get('dberror_message'));
			return(&error_page);
		}
	} elsif($current_user->userlevel_id < '40') {
		#pending change
		$pending_change = Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $rpm->id,
					'table_name' => 'rpm_requires',
					'field_name' => '_NEW_',
					'proposed_value' => $req->name,
		});
		
		Kaizen::RPMDE::PendingChanges->create({
				'user_id' => $current_user->id,
				'primary_id' => $pending_change->id,
				'table_name' => 'rpm_requires_new',
				'field_name' => 'version',
				'proposed_value' => $req->version,
		});
	
		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
		
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}
}

########################################
########################################
sub saverpm_page {
	my %cgi_var = ();
	
	# setup a hash for the rpm values
	$cgi_var{$_} = $q->param($_) foreach $q->param();
    $cgi_var{'dateadded'} = Time::Piece->new;
	$cgi_var{'dateupdated'} = Time::Piece->new;
	
	my $rpm = Kaizen::RPMDE::RPM->create(\%cgi_var);

	if($rpm) {
		$rpm->add_to_status({spec => 0,build => 0, success => 0});
	
		return("REDIRECT:/editrpm?id=" . $rpm->id);
	} else {
		return("REDIRECT:/home");
	}
}

########################################
########################################
sub savesource_page {
	my $popup_close_tpl = new HTML::Template(filename => $conf->get('template_dir') . "popup_close.tpl");
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
	my ($source,$pending_change) = '';
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {
		#save the source file
		if(&save_file($q->param("source"))) {
			my $md5 = Digest::MD5->new;
			open(FH,$conf->get('sources_dir') . '/' . $q->param('source'));
			binmode(FH);
			$md5->addfile(*FH);
			close(FH);
			
			$source = $rpm->add_to_sources({
				'source' => $q->param('source'),
				'original' => $q->param('original'),
				'include' => ($q->param('include')) ? '1' : '0',
				'size' => (-s $conf->get('sources_dir') . '/' . $q->param('source')),
				'md5sum' => $md5->hexdigest
				});
	
			&update_rpm_status($rpm,'0','0','0');
			
			if($source) {
				return($popup_close_tpl->output);
			} else {
				$q->param('message',$conf->get('dberror_message'));
				return(&error_page);
			}
		} else {
			$q->param('message',$conf->get('dberror_message'));
			return(&error_page);
		}
	} elsif($current_user->userlevel_id < '40') {
		#pending change
		if(&save_file($q->param('source'),1)) {
			$pending_change = Kaizen::RPMDE::PendingChanges->create({
						'user_id' => $current_user->id,
						'primary_id' => $rpm->id,
						'table_name' => 'rpm_source',
						'field_name' => '_NEW_',
						'proposed_value' => $q->param('source'),
			});
			
			Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $pending_change->id,
					'table_name' => 'rpm_source_new',
					'field_name' => 'original',
					'proposed_value' => $q->param('original'),});
			
			Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $pending_change->id,
					'table_name' => 'rpm_source_new',
					'field_name' => 'include',
					'proposed_value' => $q->param('include'),});
		
		}
		
		$q->param('message',$conf->get('pending_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	} else {
		$q->param('message',$conf->get('disabled_message'));
		$q->param('suppress_error',1);
		return(&error_page);
	}	
}

########################################
########################################
sub savesubpackage_page {
	my $popup_close_tpl = new HTML::Template(filename => $conf->get('template_dir') . "popup_close.tpl");
	
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
	my ($subpackage,$pending_change) = '';
	
	if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {
		#add the subpackage
		$subpackage = $rpm->add_to_subpackages({
			'name' => $q->param('name')});
		&update_rpm_status($rpm,'0','0','0');		
	
	} elsif($current_user->userlevel_id < '40') {
		$pending_change = Kaizen::RPMDE::PendingChanges->create({
					'user_id' => $current_user->id,
					'primary_id' => $rpm->id,
					'table_name' => 'rpm_subpackages',
					'field_name' => '_NEW_',
					'proposed_value' => $q->param('name'),
		});

		$q->param("message",$conf->get('pending_message'));
		$q->param("suppress_error",1);
		return(&error_page);			
	} else {
		$q->param("message",$conf->get('disabled_message'));
		return(&error_page);
	}		
		
	return($popup_close_tpl->output);
}

########################################
########################################
sub search_page {
	my $search_tpl = new HTML::Template(filename => $conf->get('template_dir') . "search.tpl");
    my $ss = $q->param("ss");
	my $total_rpms = 0;
	
	my @rpms = Kaizen::RPMDE::RPM->search_where(name => {'like','%' . $ss . '%'}) unless ($ss eq "");
	$total_rpms = Kaizen::RPMDE::RPM->count_search_like(name => '%' . $ss . '%') unless ($ss eq "");
	
	my ($total_rpm_size,$total_srpm_size) = 0;
	foreach(@rpms) {
		$total_rpm_size += $_->rpmsize;
		$total_srpm_size += $_->srpmsize;
	}
	
	$search_tpl->param(
			title => "Kaizen::RPMDE::RPMS::Search",
			menubar => &get_menubar,
			ss => $ss,
			total_rpms => $total_rpms,
			total_size => nearest(0.01,$total_rpm_size/1024/1024),
			total_ssize => nearest(0.01,$total_srpm_size/1024/1024),
			rpms => 
					[map {
						'id' => $_->id,
						'name' => $_->name,
						'version' => $_->version,
						'dateupdated' => $_->dateupdated->ymd,
						'changelog' => '',
						'packager' => $_->packager_id->name,
						'packager_email' => $_->packager_id->email,
						'website' => $_->url,
						'bgcolor' => $_->bgcolor,
						'rpm' => $_->rpmname,
						'srpm' => $_->srpmname,
						'rpmsize' => $_->getrpmsize,
						'srpmsize' => $_->getsrpmsize,},
					sort {$a->name cmp $b->name}
					map {$_} @rpms]);
	
	return($search_tpl->output);
}

########################################
########################################
sub updaterpm_page {
	my (%cgi_var,%subpackage) = ();
	my @temp = ();
	my $subpack = "";
	
	$cgi_var{$_} = $q->param($_) foreach $q->param();
	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));

	if($rpm) {
		#the subpackages defined will pass their values as name-field
		#let's break them out into the subpackage hash
		foreach(keys(%cgi_var)) {
			if($_ =~ /-/) {
				@temp = split(/-/,$_);
				$subpackage{$temp[0]}{$temp[1]} = $cgi_var{$_};
	
				delete($cgi_var{$_});
			}
		}
						
		if($current_user->userlevel_id eq '10' || ($current_user->userlevel_id eq '20' && $current_user->id eq $rpm->packager_id)) {
			#loop through the subpackages and save the updated information
			foreach(keys(%subpackage)) {
				($subpack) = Kaizen::RPMDE::RPMSubpackages->search(name => $_, rpm_id => $q->param('id'));
				
				$subpack->set(%{$subpackage{$_}});
				$subpack->update;
			}
				
			$rpm->set(%cgi_var);
			$rpm->set(dateupdated => Time::Piece->new);
			
			#let's automatically output the spec file here
			&output_spec($rpm);
			
			&update_rpm_status($rpm,'1','0','0');
			$rpm->update;
		
			return("REDIRECT:/editrpm?id=" . $q->param('id'));
		} elsif($current_user->userlevel_id < 40) {
			#add to the pending changes
			my $pending_change = "";
			
			foreach(keys(%cgi_var)) {
				#only make the change if the value has changed
				$log->write('checking ' . $rpm->$_ . ' against ' . $cgi_var{$_} . ' for field ' . $_);
				if($rpm->$_ ne $cgi_var{$_}) { 
					$pending_change = Kaizen::RPMDE::PendingChanges->create({
								'user_id' => $current_user->id,
								'primary_id' => $rpm->id,
								'table_name' => 'rpm',
								'field_name' => $_,
								'proposed_value' => $cgi_var{$_},
					});
				}
			}
			
			#loop through the subpackages and check for required updates
			foreach(keys(%subpackage)) {
				($subpack) = Kaizen::RPMDE::RPMSubpackages->search(name => $_, rpm_id => $q->param('id'));
				
				foreach my $k (keys(%{$subpackage{$_}})) {
					$log->write('checking ' . $subpack->$k . ' against ' . $subpackage{$_}{$k} . ' for field ' . $k);
					if($subpack->$k ne $subpackage{$_}{$k}) {
						#create a new pending option
						$pending_change = Kaizen::RPMDE::PendingChanges->create({
									'user_id' => $current_user->id,
									'primary_id' => $subpack->id,
									'table_name' => 'rpm_subpackages',
									'field_name' => $k,
									'proposed_value' => $subpackage{$_}{$k},
						});
					}
				}
			}			
			
			$q->param("message",$conf->get('pending_message'));
			$q->param("suppress_error",1);
			return(&error_page);
		} else {
			$q->param("message",$conf->get('disabled_message'));
			return(&error_page);
		} 
	} else {
		return("REDIRECT:/managerpm");
	}
}

########################################
########################################
sub updateyum_page {
	my $updateyum_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "updateyum.tpl");
	my $server_id = $q->param("server_id");
	my $count = 0;
	my (@ftpservers,@ftpmenu,@rpms) = ();
		    
	#get the list of ftp servers
	if(!defined $server_id || $server_id eq "") {
		@ftpmenu = map {
						'id' => $_->id,
						'name' => $_->name,},
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::SysFTPServers->retrieve_all;
	} else {
		@ftpservers = map {
						'id' => $_->id,
						'name' => $_->name,
						'dateupdated' => $_->dateupdated->ymd,
						'basedir' => $_->basedir},
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::SysFTPServers->retrieve($server_id);
							
		@ftpmenu = map {
						'id' => $_->id,
						'name' => $_->name,},
					sort {$a->name cmp $b->name}
					map {$_} Kaizen::RPMDE::SysFTPServers->retrieve($server_id);
	}	
	
	foreach(@ftpservers) {
		@rpms = Kaizen::RPMDE::RPM->search_where('dateupdated' => {'>=',$ftpservers[$count]{'dateupdated'}});
		
		foreach(@rpms) {
			if($_->rpmname ne "") {
				push @{$ftpservers[$count]{'rpms'}},map {
												'id' => $_->id,
												'ftpid' => $ftpservers[$count]{'id'},
												'name' => $_->name,
												'size' => $_->rpmsize + $_->srpmsize,
												'bgcolor' => $_->bgcolor,
												'dateupdated' => $_->dateupdated->ymd,
												'destination' => $ftpservers[$count]{'basedir'} . $_->getrpmdir},
												map {$_} $_;
			}		
		}
		
		delete($ftpservers[$count]{'basedir'});
		delete($ftpservers[$count]{'id'});
		$count++;
	}	
			
	$updateyum_tpl->param(
						title => "Kaizen::RPMDE::UpdateYum",
						menubar=>&get_menubar,
						ftpservers=>\@ftpservers,
						ftpmenu=>\@ftpmenu,
						);
	
	return($updateyum_tpl->output);
}

########################################
########################################
sub updateyum_doit_page {
	my $updateyum_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "updateyum.tpl");
	my $header_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "header.tpl");
	
    my ($server,$rpm,$results,$ftp,$flag,$file) = "";
	my ($command,$output) = "";
	my (@ls,%updates,@subpackages) = ();
		
	#need to setup a hash of arrays to hold our update data
	foreach($q->param("updates")) {
		($server,$rpm) = split(/:/,$_);
		
		$flag = 1;
		foreach(@{$updates{$server}}) {
			if($_ eq $rpm) {
				$flag = 0;
			}
		}
		
		if($flag) {
			push @{$updates{$server}},$rpm;
		}
	}
	
	#execute yum-arch for base and updates
	$command = "sudo yum-arch /var/www/rms/rpm/base/1.0beta/i386";
	$output .= qx($command);
	
	$command = "sudo yum-arch /var/www/rms/rpm/updates/1.0beta/i386";
	$output .= qx($command);
	
	$header_tpl->param(menubar=>&get_menubar);
		
	$r->content_type('text/html');
	$r->print($header_tpl->output);
	$r->print("<b>Updating yum repositories...</b><br><br>");
	
	if($output =~ /Writing header.info file/) {
		$r->print("<font color='green'><b>Wrote header.info file</b></font><br>br>");
	} else {
		$r->print("<font color='red'>ERROR: Could not write header.info file</font><br>br>");
	}
	
	foreach(keys %updates) {
		$server = Kaizen::RPMDE::SysFTPServers->retrieve($_);
		
		$r->print("<b>Connecting to " . $server->name . "...</b><BR><BR>");
		
		#connect and login to our ftp server
		$ftp = Net::FTP->new($server->host);
		$ftp->login($server->username,$server->password) or $results = "Error: " . $ftp->message;
		
		$r->print("<b>Switching to binary mode...</b><br><br>");
		$ftp->binary;
		
		foreach(@{$updates{$_}}) {
			$rpm = Kaizen::RPMDE::RPM->retrieve($_);
			
			$r->print("Uploading RPMs for package " . $rpm->name . "...<BR>");
			
			#upload the rpm
			$r->print("<i>sending /var/www/rms/rpm/" . $rpm->getrpmdir . $rpm->rpmname . " to " . $server->basedir . $rpm->getrpmdir . $rpm->rpmname . ".....");
			$file = $ftp->put("/var/www/rms/rpm/" . $rpm->getrpmdir . $rpm->rpmname,$server->basedir . $rpm->getrpmdir . $rpm->rpmname);
			if($file eq $server->basedir . $rpm->getrpmdir . $rpm->rpmname) {
				$r->print("<font color='green'>SUCCESS</font></i><br>");
			} else {
				$r->print("<font color='red'>FAILURE - " . $ftp->message . "</font></i><br>");
			}
			
			#upload the header for the rpm
			$r->print("<i>sending /var/www/rms/rpm/" . $rpm->getrpmheaderdir . $rpm->rpmheadername . " to " . $server->basedir . $rpm->getrpmheaderdir . $rpm->rpmheadername . ".....");
			$file = $ftp->put("/var/www/rms/rpm/" . $rpm->getrpmheaderdir . $rpm->rpmheadername,$server->basedir . $rpm->getrpmheaderdir . $rpm->rpmheadername);
			if($file eq $server->basedir . $rpm->getrpmheaderdir . $rpm->rpmheadername) {
				$r->print("<font color='green'>SUCCESS</font></i><br>");
			} else {
				$r->print("<font color='red'>FAILURE - " . $ftp->message . "</font></i><br>");
			}
						
			#check for any subpackages to move as well
			@subpackages = Kaizen::RPMDE::RPMSubpackages->search(rpm_id=>$rpm->id);
			
			foreach(@subpackages) {
				$r->print("<i>sending /var/www/rms/rpm/" . $rpm->getrpmdir . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm' . " to " . $server->basedir . $rpm->getrpmdir . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm' . ".....");
				$file = $ftp->put("/var/www/rms/rpm/" . $rpm->getrpmdir . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm',$server->basedir . $rpm->getrpmdir . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm');
				if($file eq $server->basedir . $rpm->getrpmdir . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm') {
					$r->print("<font color='green'>SUCCESS</font></i><br>");
				} else {
					$r->print("<font color='red'>FAILURE - " . $ftp->message . "</font></i><br>");
				}
			
				$r->print("<i>sending /var/www/rms/rpm/" . $rpm->getrpmheaderdir . $rpm->name . '-' . $_->name . '-' . $rpm->release . '-' . $rpm->version . '-' . $rpm->release . '.i386.hdr' . " to " . $server->basedir . $rpm->getrpmheaderdir . $rpm->name . '-' . $_->name . '-' . $rpm->release . '-' . $rpm->version . '-' . $rpm->release . '.i386.hdr' . ".....");
				$file = $ftp->put("/var/www/rms/rpm/" . $rpm->getrpmheaderdir . $rpm->name . '-' . $_->name . '-' . $rpm->release . '-' . $rpm->version . '-' . $rpm->release . '.i386.hdr',$server->basedir . $rpm->getrpmheaderdir . $rpm->name . '-' . $_->name . '-' . $rpm->release . '-' . $rpm->version . '-' . $rpm->release . '.i386.hdr');
				if($file eq $server->basedir . $rpm->getrpmheaderdir . $rpm->name . '-' . $_->name . '-' . $rpm->release . '-' . $rpm->version . '-' . $rpm->release . '.i386.hdr') {
					$r->print("<font color='green'>SUCCESS</font></i><br>");
				} else {
					$r->print("<font color='red'>FAILURE - " . $ftp->message . "</font></i><br>");
				}
			}					
			
			#upload the srpm
			$r->print("<i>sending /var/www/rms/rpm/" . $rpm->getsrpmdir . $rpm->srpmname . " to " . $server->basedir . $rpm->getsrpmdir . $rpm->srpmname . ".....");
			$file = $ftp->put("/var/www/rms/rpm/" . $rpm->getsrpmdir . $rpm->srpmname,$server->basedir . $rpm->getsrpmdir . $rpm->srpmname);
			if($file eq $server->basedir . $rpm->getsrpmdir . $rpm->srpmname) {
				$r->print("<font color='green'>SUCCESS</font></i><br>");
			} else {
				$r->print("<font color='red'>FAILURE - " . $ftp->message . "</font></i><br>");
			}			
		}
		
		$r->print("<br><br>Uploading header.info files...<br>");
		
		#upload the header.info file for the base and updates
		$r->print("<i>sending /var/www/rms/rpm/base/1.0beta/i386/headers/header.info to " . $server->basedir . "base/1.0beta/i386/headers/header.info.....");
		$file = $ftp->put("/var/www/rms/rpm/base/1.0beta/i386/headers/header.info",$server->basedir . "base/1.0beta/i386/headers/header.info");
		if($file eq $server->basedir . "base/1.0beta/i386/headers/header.info") {
			$r->print("<font color='green'>SUCCESS</font></i><br>");
		} else {
			$r->print("<font color='red'>FAILURE - " . $ftp->message . "</font></i><br>");
		}

		$r->print("<i>sending /var/www/rms/rpm/updates/1.0beta/i386/headers/header.info to " . $server->basedir . "updates/1.0beta/i386/headers/header.info.....");
		$file = $ftp->put("/var/www/rms/rpm/updates/1.0beta/i386/headers/header.info",$server->basedir . "updates/1.0beta/i386/headers/header.info");
		if($file eq $server->basedir . "updates/1.0beta/i386/headers/header.info") {
			$r->print("<font color='green'>SUCCESS</font></i><br>");
		} else {
			$r->print("<font color='red'>FAILURE - " . $ftp->message . "</font></i><br>");
		}
			
		#disconnect from the server                                              
		$r->print("<BR><b>Disconnecting from " . $server->name . "...</b><BR><BR>");
		
		#update the server
		$server->set(dateupdated => Time::Piece->new);
		$server->update;
	}
	
	#disconnect from the server
	$ftp->quit;
		
	return("IGNORE");
}

########################################
########################################
sub viewrpm_page {
	my $newrpm_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "newrpm.tpl");

	my $rpm = Kaizen::RPMDE::RPM->retrieve($q->param('id'));
		
	my @rpms = 
		({'view' => 1, 'id' => $rpm->id,
		'name' => $rpm->name, 'version' => $rpm->version,
		'release' => $rpm->release, 'description' => $rpm->description,
		'summary' => $rpm->summary,
		'copyright_name' => $rpm->sys_copyright_id->name . " - " . $rpm->sys_copyright_id->description,
		'distribution_name' => $rpm->sys_distribution_id->name,
		'icon' => $rpm->icon, 'vendor' => $rpm->vendor,
		'url' => $rpm->url, 'group_name' => $rpm->sys_group_id->name,
		'packager_name' => $rpm->packager_id->name, 'serial' => $rpm->serial,
		'autoreqprov' => $rpm->autoreqprov, 'provides' => $rpm->provides,
		'excludearch' => $rpm->excludearch, 'exclusivearch' => $rpm->exclusivearch,
		'excludeos' => $rpm->excludeos, 'exclusiveos' => $rpm->exclusiveos,
		'prefix' => $rpm->prefix, 'buildroot' => $rpm->buildroot,
		'defines' => $rpm->defines, 'prep' => $rpm->prep, 'build' => $rpm->build,
		'install' => $rpm->install, 'clean' => $rpm->clean,
		'pre' => $rpm->pre, 'post' => $rpm->post,
		'preun' => $rpm->preun, 'postun' => $rpm->postun,
		'verifyscript' => $rpm->verifyscript, 'files' => $rpm->files,
		'view' => 1,
		'changelog' => 
			[map {'date' => ('* ' . $_->date->day . ' ' . $_->date->month . ' '  . $_->date->mday
							. ' ' . $_->date->year . ' ' . $rpm->packager_id->name
							. ' &lt;' . $rpm->packager_id->email . '&gt;'),
				  'description' => sub_newline($_->description),},
				  $rpm->changelogs],
		'sources' => 
			[map {'view' => 1,
				  'source' => $_->source,
				  'sourcedir' => '/rpm/SOURCES/',
				  'include' => ($_->include) ? '1' : '',
				  'original' => $_->original,
				  'size' => nearest(0.01,(-s File::Spec->catfile($ENV{DOCUMENT_ROOT},"rpm","SOURCES",$_->source))/1024/1024),},
				$rpm->sources],
		'patches' => 
			[map {'view' => 1,
				  'patch' => $_->patch,
				  'patchdir' => '/rpm/SOURCES/',
				  'include' => ($_->include) ? '1' : '',
				  'size' => nearest(0.01,(-s File::Spec->catfile($ENV{DOCUMENT_ROOT},"rpm","SOURCES",$_->patch))/1024),},
				$rpm->patches],
		'requires' =>
			[map {'name' => $_->name,
				  'version' => $_->version},
			$rpm->requires],
		'subpackages' =>
			[map {'id' => $_->id,
				  'name' => $_->name,
				  'summary' => $_->summary,
				  'description' => $_->description,
				  'group_name' => $_->sys_group_id->name,
				  'pre' => $_->pre,
				  'post' => $_->post,
				  'preun' => $_->preun,
				  'postun' => $_->postun,
				  'verifyscript' => $_->verifyscript,
				  'files' => $_->files,
				  'view' => 1,
				  },
			$rpm->subpackages],
		'subpackages_menu' =>
			[map {'name' => $_->name,},
			$rpm->subpackages],
		});
							
	$newrpm_tpl->param(
			title => "Kaizen::RPMDE::RPMS::View",
			menubar => &get_menubar,
			view => 1,
			id => $q->param('id'),
			website => $rpm->url,
			rpmdata => \@rpms);
    
	return($newrpm_tpl->output);
}

########################################
# PRIVATE subroutines
# this is a placeholder for my subroutine list
########################################
sub _____PRIVATE____ {}
######################################
# name: check_cookie
# description: verifies a passed cookie is a valid user
# parameters: none
# return values: true/false
######################################
sub check_cookie {
	my @value = ();
	my $credentials = "";
	my @user = ();

	$credentials = $q->cookie('credentials');

	if(defined $credentials) {
		$log->write("cookie is found: $credentials",5);
		@value = split(/:/,$credentials);

		@user = Kaizen::RPMDE::Users->retrieve_from_sql("username='" . $value[1] . "' AND password='" . $value[3] . "'");

		if(@user) {
			$current_user = $user[0];
			
			$log->write("Current user is: " . $current_user->username);
			return(1);
		}
	}
	
	$log->write("User failed authentication -- redirecting to login page");

	return(0);
}

######################################
# name: get_foldersize
# description: returns the size of a folder
# parameters: directory
# return values: block size
######################################
sub get_foldersize {
     local $% = 0;
     find( { wanted => sub {
          local ($-,$=) = (-s,(stat)[11]);
          $%+=(int($-/$=)+($-%$=?1:0))*$=
     }},shift);$%
}

######################################
# name: get_menubar
# description: returns the menubar array of hashes 
# parameters: none
# return values: menubar array of hashes
######################################
sub get_menubar {
	my (@menubar,@split) = ();
	my ($data,$count) = "";

	#get the menubars
	@menubar = Kaizen::RPMDE::SysMenu->retrieve_from_sql(qq{1=1 ORDER BY menu_order});
	
	@split = split(/_/,$path_t);
    if($split[0] eq 'search' || $split[0] eq 'managerpm' || $split[0] eq 'newrpm'
		|| $split[0] eq 'viewrpm' || $split[0] eq 'generatespec' || $split[0] eq 'buildrpm'
		|| $split[0] eq 'editrpm' || $split[0] eq 'executerpm') {
		$split[0] = 'rpms';
	}
	
	return([map {
		'name' => $_->name,
		'href' => $_->href,
		'selected' => ($_->href eq '/' . $split[0]) ? '-selected':''}, @menubar]);
}

######################################
# name: get_todays_date
# description: returns todays date in (yyyy/mm/dd) format
# parameters: offset (in days)
# return values: date string
######################################
sub get_todays_date() {
	my ($offset) = @_;
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	my ($b1,$b2,$date,$y,$m,$d);
	
	($y,$m,$d) = Add_Delta_Days($year,$mon,$mday,$offset);

	return($date);
}

######################################
# name: get_user_cookie
# description: returns a cookie for the user
# parameters: username, password
# return values: cookie
######################################
sub get_user_cookie {
	my ($username,$password) = @_;
	my $c = "";

	my @user = Kaizen::RPMDE::Users->retrieve_from_sql("username='$username' AND password=PASSWORD('$password')");
	
	if(@user) {
		$log->write("User Found -- generating cookie");
		
		$c = $q->cookie(
				-name			=>'credentials',
				-value			=>"username:" . $user[0]->username . ":password:" . $user[0]->password,
			    -expires		=>'',
			    -domain			=>$conf->get('domain'),
				-path			=>'/');
	}

	$log->write("returning cookie value: $c",5);

	return($c);
}

######################################
# name: output_spec
# description: outputs a spec file for a specific rpm
# parameters: rpm object
# return values: true|false
######################################
sub output_spec {
	my ($rpm) = @_;
	my $output_spec_tpl = new HTML::Template(filename=>$conf->get('template_dir') . "output_spec.tpl");
	my ($c_source,$c_patch) = 0;
	my $spec_file = "";
	
	my @rpms = 
		({'name' => $rpm->name, 'version' => $rpm->version,
		'release' => $rpm->release, 'description' => $rpm->description,
		'summary' => $rpm->summary,
		'copyright_name' => $rpm->sys_copyright_id->name,
		'distribution_name' => $rpm->sys_distribution_id->name, 'icon' => $rpm->icon, 
		'vendor' => $rpm->vendor, 'url' => $rpm->url, 'group_name' => $rpm->sys_group_id->name,
		'packager_name' => $rpm->packager_id->name, 'packager_email' => $rpm->packager_id->email,
		'serial' => $rpm->serial, 'autoreqprov' => $rpm->autoreqprov, 
		'excludearch' => $rpm->excludearch, 'exclusivearch' => $rpm->exclusivearch,
		'excludeos' => $rpm->excludeos, 'exclusiveos' => $rpm->exclusiveos,
		'prefix' => $rpm->prefix, 'buildroot' => $rpm->buildroot, 'defines' => $rpm->defines,
		'prep' => $rpm->prep, 'build' => $rpm->build, 'install' => $rpm->install, 
		'clean' => $rpm->clean,	'pre' => $rpm->pre, 'post' => $rpm->post, 'preun' => $rpm->preun, 
		'postun' => $rpm->postun, 'verifyscript' => $rpm->verifyscript, 'files' => $rpm->files,
		'provides' =>
			[map {'value' => $_},split(/\n/,$rpm->provides)],
		'changelog' => 
			[map {'date' => ('* ' . $_->date->day . ' ' . $_->date->month . ' '  . $_->date->mday
							. ' ' . $_->date->year . ' ' . $rpm->packager_id->name
							. ' &lt;' . $rpm->packager_id->email . '&gt;'),
				  'description' => $_->description,},
				  $rpm->changelogs],
		'sources' => 
			[map {'source' => $_->original . $_->source,
				  'count' => $c_source++,},
				  $rpm->sources],
		'patches' => 
			[map {'patch' => $_->patch,
				  'count' => $c_patch++,},
				  $rpm->patches],
		'requires' =>
			[map {'name' => $_->name,
				  'version' => $_->version},
			$rpm->requires],
		'subpackages' =>
			[map {'name' => $_->name,
				  'summary' => $_->summary,
				  'description' => $_->description,
				  'group_name' => $_->sys_group_id->name,
				  'pre' => $_->pre,
				  'post' => $_->post,
				  'preun' => $_->preun,
				  'postun' => $_->postun,
				  'verifyscript' => $_->verifyscript,
				  'files' => $_->files,
				  },
			$rpm->subpackages],
		});

	#open the file for output
	open(SPEC,">" . $conf->get('specs_dir') . "/" . $rpm->name . "-" . $rpm->version . "-" . $rpm->release . ".spec");
	
	#setup the template for the spec file
	$output_spec_tpl->param(
		rpmdata => \@rpms);

	#write the spec file to disk		
	$spec_file = $output_spec_tpl->output;

	#get rid of most of the newlines and ^M's
	$spec_file =~ s/[\n]+/\n/gs;
	$spec_file =~ s/\cM//gs;
	
	#rpm tags
	foreach(@RPM_NEWLINE_TAGS) {
		$log->write("processing $_");
		$spec_file =~ s/$_/\n$_/g;
	}
	
	#ouput the text to the file
	print SPEC $spec_file;
	
	#close the spec file
	close(SPEC);
}

######################################
# name: save_file
# description: stores an uploaded file
# parameters: document that was uploaded (essentially file param from posted data)
# return values: 
######################################
sub save_file {
	my ($file,$temp) = @_;
	my ($filename,$u_filename) = "";
	
	#let's create the filename with File::Spec so it should work under any environment
	if($temp) {
		$filename = File::Spec->catfile($ENV{DOCUMENT_ROOT},'rpm','TMPSRC',$file);
	} else {
		$filename = File::Spec->catfile($ENV{DOCUMENT_ROOT},'rpm','SOURCES',$file);
	}
	
	#untaint the filename
	$filename =~ /([a-zA-Z0-9\-\\:\/\.\_\+]+)/;
	$u_filename = $1;
	
	$log->write("writing file to disk: $u_filename");
	
	#write the file to disk
	open(FH,">$u_filename");
	binmode(FH);
	while(<$file>) {
		print FH $_;
	}
	close(FH);
	
    #let's make sure the file exists
    if(-e $u_filename) {
		return(1);
	}
	
	return(0);
}

######################################
# name: send_email
# description: sends an email
# parameters:
#					to_email
#					from_email
#					subject
#					body
#					bcc_email
# return values: none
######################################
sub send_email {
	my $sender = new Mail::Sender({smtp => $conf->get('smtp_server')});
	my ($to,$from,$subject,$body,$bcc) = @_;

	$log->write("sending email to $to");

	$sender->OpenMultipart({
			to => $to,
			from => $from,
			bcc => $bcc,
			subject => $subject});
	$sender->Body({
			msg => $body . "\n\n"});
	$sender->Close;

	undef($sender);
}

######################################
# name: sub_newline
# description: returns the same string with newlines converted to <br>'s
# parameters: string
# return values: modified string
######################################
sub sub_newline {
	my ($string) = @_;
	
	$string =~ s/\n/<br>/g;
	
	return($string);
}

######################################
# name: update_rpm_status
# description: updates an rpms status
# parameters: rpm object, spec status, (build, success, in_progress)
#			  (at least one of spec or build status)
# return values: true/false
######################################
sub update_rpm_status {
	my ($rpm,$spec,$build,$success,$in_progress) = @_;
	
	$log->write("inside update_rpm_status");
	$log->write("received rpm value: $rpm");
	
	my ($status) = $rpm->status;
	
	$log->write("retrieved status value: $status");
	
	if($status) {
		$status->set(
			spec => ($spec) ? $spec : $status->spec,
			build => ($build) ? $build : $status->build,
			success => ($success) ? $success : $status->success,
			in_progress => ($in_progress) ? $in_progress : $status->in_progress,
			);
			
		$status->update;
	}
	
	return(1);
}

########################################
# Necessary!  DO NOT REMOVE!
########################################
1;
