#!/usr/bin/perl

# set this directory to the location of the common modules
use lib '/path/to/common-modules-1.0/';



#****************************************************
# You do not need to edit anything beyond this point
#****************************************************


################################
# required modules
################################
use POSIX;
use Log::LogLite;
use IO::Handle;
use Net::FTP;
use Digest::MD5;
use Time::Piece;
use File::Copy;

# class::dbi modules
use Kaizen::RPMDE::DBI;
use Kaizen::RPMDE::RPM;
use Kaizen::RPMDE::RPMBuildQueue;
use Kaizen::RPMDE::RPMChangelog;
use Kaizen::RPMDE::RPMConflicts;
use Kaizen::RPMDE::RPMPatch;
use Kaizen::RPMDE::RPMRequires;
use Kaizen::RPMDE::RPMSource;
use Kaizen::RPMDE::RPMStatus;
use Kaizen::RPMDE::RPMSubpackages;
use Kaizen::RPMDE::SysBuilds;
use Kaizen::RPMDE::SysCopyright;
use Kaizen::RPMDE::SysDistribution;
use Kaizen::RPMDE::SysGroup;
use Kaizen::RPMDE::SysMenu;
use Kaizen::RPMDE::Users;

################################
# setup constants
################################
use constant VERSION			=> "1.0";
use constant LOG_FILE			=> '/var/log/rpmde_daemon.log';
use constant LOG_LEVEL			=> '2';		# 1 = Debug Messages
											# 2 = Commands executed
											# 3 = Include output of commands
#setup chroot vars
use constant CHROOT				=> "/mnt/mel";
use constant CHROOT_CMD			=> 'chroot /mnt/mel /usr/bin/env -i HOME=/root '
									. 'PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/java/jdk1.5.0_01/bin:/usr/X11R6/bin/ ';

#ftp constants
use constant CACHE_DIR			=> '/var/cache/rpmde/';
use constant SERVER_HOST		=> 'HOSTNAME';
use constant SERVER_USERNAME	=> 'USERNAME';
use constant SERVER_PASSWORD	=> 'PASSWORD'; 
use constant SERVER_BASE			=> "/www/kaizen/rpmde/rpm/";

################################
# setup some vars
################################
$time_to_die = 0;
$building = 0;
$first_flag = 1;
$log = new Log::LogLite(LOG_FILE, LOG_LEVEL);
$log->write("*** rpmde_daemon started",1);

################################
# trap signals
################################
$SIG{INT} = $SIG{TERM} = $SIG{HUP} = \&DIE_GRACEFUL;
$SIG{PIPE} = 'IGNORE';
$SIG{CHLD} = \&REAPER;

sub DIE_GRACEFUL {
	#remove the pid file
	system("rm /var/run/rpmde_daemon.pid") if (-e '/var/run/rpmde_daemon.pid');
	$time_to_die = 1;
}

sub REAPER {
	1 until (-1 == waitpid(-1,WNOHANG));
	$SIG{CHLD} = \&REAPER;
}

################################
# detach from parent
################################
$log->write("starting to detach from parent",1);

$pid = fork;
$log->write("pid after fork is $pid",1);
if($pid) {
	#see if we're already running
	if(-e '/var/run/rpmde_daemon.pid') {
		$log->write("we're already running",1);
		print "ERROR: I appear to already be running. Check /var/run/rpmde_daemon.pid\n";
		
		kill 9 => $pid;
	} else {
		#output the pid file
		$log->write('writing pid to /var/run/rpmde_daemon.pid',1);
		open(PID,">/var/run/rpmde_daemon.pid");
		print PID $pid;
		close(PID);
	}
	exit;
} 

die "Couldn't fork: $!" unless defined($pid);
POSIX::setsid() or die "Can't start a new session: $!";

$log->write("finished detaching from parent",1);
	
################################
# start main loop
################################
$log->write("starting main loop",1);
until($time_to_die) {
	#start our communicator
	pipe(READER,WRITER);
	WRITER->autoflush(1);
	
	if($pid = fork) {
		$log->write("pid after fork is $pid",1);
		@builds = Kaizen::RPMDE::RPMBuildQueue->retrieve_from_sql("status = 'waiting'");
		
		#parent process - check for something to build
		$count=0;
		if(@builds) {
			$log->write("found some builds",1);
			while($builds[$count]->status eq 'building') {$count++;}
			close READER;
			print WRITER $builds[$count]->rpm_id . ':' . $builds[$count]->build_type . ':' . $builds[$count]->id;
			close WRITER;
			
			#ugly hack, but it gets the job done...
			my $temp_build = $builds[$count];
			$temp_build->set(status=>'working');
			$temp_build->update;
			
			$log->write("starting to build " . $builds[$count]->rpm_id,1);
			waitpid($pid,0);
			$log->write("finished building " . $builds[$count]->rpm_id,1);
			$builds[$count]->set(status=>'done');
			$builds[$count]->update;
		} else {
			close READER;
			print WRITER 'NOCMD';
			close WRITER;
			
			waitpid($pid,0);

			if($first_flag) {
				#sleep for 30 seconds the first time around
				$log->write("sleeping for 30 seconds",1);
				sleep(30);
				$first_flag = 0;
			} else {
				#sleep for 3 minutes before checking for another build
				$log->write("sleeping for 300 seconds",1);
				sleep(300);
			}
		}		
	} else {
		#child process
		$log->write("inside the child process",1);
		close WRITER;
		chomp($line = <READER>);
		$log->write("Command received from parent: $line",1);
		
		#setup some variables for the build
		my ($temp_output,$md5sum,$dl_attempts,$fail_message,$chroot_dir,$file_dir) = "";
		my ($output,$output_log,$ftp) = "";
		my %rpminfo = ();
		my @subpackageinfo = ();
		
		my $md5 = Digest::MD5->new;
		my $date = Time::Piece->new;
		
		if($line ne 'NOCMD' || $line eq "") {
			$log->write("we got a command -- let's get to work",1);
			my ($id,$build_type,$queue_id) = split(/:/,$line);
			my $rpm = Kaizen::RPMDE::RPM->retrieve($id);
    		my $build = Kaizen::RPMDE::SysBuilds->retrieve($build_type);
			my $queue = Kaizen::RPMDE::RPMBuildQueue->retrieve($queue_id);
	
			my ($command,$temp_file,$status) = "";
	                                                                              
			if($rpm) {
				$log->write("starting to build rpm: " . $rpm->name,1);

				#setup the users chroot directory
				$chroot_dir = '/mnt/' . $queue->user_id->chroot_dir . '/';
				$file_dir = '/mnt/' . $queue->user_id->chroot_dir . '/usr/src/packages/';
				$output_log = $rpm->name . $date->strftime("%Y-%m-%d-%H-%M-%S") . '.log';
				$output = new Log::LogLite('/var/tmp/' . $output_log);
				
				#write spec file to disk if we need to
   				($status) = $rpm->status;
		
				if(! $status->spec) {
					$fail_message = "SPEC file hasn't been written.\n\nTry regenerating the spec file and schedule your build again.";
					goto FAIL;
				}
				
				#update the rpm status to "in progress"
				$status->set(
						build => '0',
						success => '0',
						in_progress => '1',
						);
				$status->update;
				
				#connect and login to our ftp server
				$ftp = Net::FTP->new(SERVER_HOST);
				$ftp->login(SERVER_USERNAME,SERVER_PASSWORD);
		
				#set binary mode
				$ftp->binary;

				#download our sources (if necessary)
				foreach($rpm->sources) {
					$md5sum = "";
					$dl_attempts = 0;
					
					while($_->md5sum ne $md5sum) {
						if(!-e CACHE_DIR . 'SOURCES/' . $_->source) {
							$log->write($_->source . " NOT FOUND in cache so I will download it now.",1);
							$ftp->get(SERVER_BASE . 'SOURCES/' . $_->source,CACHE_DIR . 'SOURCES/' . $_->source);
						} else {
							$log->write($_->source . " FOUND in cache so no downloading is necessary.",1);
						}
						
						$log->write("checking md5sum for " . $_->source,1);
						open(FH,CACHE_DIR . 'SOURCES/' . $_->source);
						binmode(FH);
						$md5->addfile(FH);
						$md5sum = $md5->hexdigest;
						
						if($md5sum ne $_->md5sum) {
							$log->write("md5sum for " . $_->source . " doesn't match the one on file.",1);
							unlink(CACHE_DIR . 'SOURCES/' . $_->source);
							$dl_attempts++;
						}
						
						if($dl_attempts eq 3) {
							$fail_message = "Couldn't download all of the sources.\n\nPlease check the md5sum's of the sources to ensure they match";
							goto FAIL;
						}
					}
				}

				#download our patches (if necessary)				
				foreach($rpm->patches) {
					$md5sum = "";
					$dl_attempts = 0;
					
					while($_->md5sum ne $md5sum) {
						if(!-e CACHE_DIR . 'SOURCES/' . $_->patch) {
							$log->write($_->patch . " NOT FOUND in cache so I will download it now.",1);
							$ftp->get(SERVER_BASE . 'SOURCES/' . $_->patch,CACHE_DIR . 'SOURCES/' . $_->patch);
						} else {
							$log->write($_->patch . " FOUND in cache so no downloading is necessary.",1);
						}
						
						$log->write("checking md5sum for " . $_->patch,1);
						open(FH,CACHE_DIR . 'SOURCES/' . $_->patch);
						binmode(FH);
						$md5->addfile(FH);
						$md5sum = $md5->hexdigest;
						
						if($md5sum ne $_->md5sum) {
							$log->write("md5sum for " . $_->patch . " doesn't match the one on file.",1);
							unlink(CACHE_DIR . 'SOURCES/' . $_->patch);
							$dl_attempts++;
						}
						
						if($dl_attempts eq 3) {
							$fail_message = "Couldn't download all of the patches.\n\nPlease check the md5sum's of the patches to ensure they match";
							goto FAIL;
						}
					}
				}
				
				#if the spec file exists, let's remove it
				if(-e CACHE_DIR . 'SPECS/' . $rpm->getrpmspecname) {
					unlink(CACHE_DIR . 'SPECS/' . $rpm->getrpmspecname);
					$log->write("Removing " . $rpm->getrpmspecname . " from cache directory",1);
				}					
				
				#download the spec file (no md5sum for the spec)
				$dl_attempts = 0;				
				while(!-e CACHE_DIR . 'SPECS/' . $rpm->getrpmspecname && $dl_attempts ne 3) {
					$log->write("starting to download the spec file -- attempt #$dl_attempts",1);
					$ftp->get(SERVER_BASE . 'SPECS/' . $rpm->getrpmspecname,CACHE_DIR . 'SPECS/' . $rpm->getrpmspecname);
					$dl_attempts++;
				}

				#disconnect from the server
				$ftp->quit;
				
				$ENV{PATH} = '/bin:/usr/sbin:/sbin';
				
				#copy the sources over to the chroot environment
				foreach($rpm->sources) {
					#if the file already exists in our chroot environment, let's delete it
					if(-e $file_dir . 'SOURCES/' . $_->source) {
						unlink($file_dir . 'SOURCES/' . $_->source);
						$log->write("Removed " . $_->source . " from your build environment",1);
					}		
					
					#copy the file
					copy(CACHE_DIR . 'SOURCES/' . $_->source,$file_dir . 'SOURCES/');
					$log->write("Copied " . $_->source . " to build environment.",1);
				}
	
				#copy the patches over to the chroot environment
				foreach($rpm->patches) {
					#if the file already exists in our chroot environment, let's delete it
					if(-e $file_dir . 'SOURCES/' . $_->patch) {
						unlink($file_dir . 'SOURCES/' . $_->patch);
						$log->write("Removed " . $_->patch . " from your build environment",1);
					}		
					
					#copy the file
					copy(CACHE_DIR . 'SOURCES/' . $_->patch,$file_dir . 'SOURCES/');
					$log->write("Copied " . $_->patch . " to build environment.",1);
				}
	
				#if the spec file already exists in our chroot environment, let's delete it
				if(-e $file_dir . 'SPECS/' . $rpm->getrpmspecname) {
					unlink($file_dir . 'SPECS/' . $rpm->getrpmspecname);
					$log->write("Removing " . $rpm->getrpmspecname . " from build directory",1);
				}					
					
				#copy the spec file over to the chroot environment
				copy(CACHE_DIR . 'SPECS/' . $rpm->getrpmspecname,$file_dir . 'SPECS/');
				$log->write("Copied " . $rpm->getrpmspecname . " to build directory",1);
	
				#build the rpm
				$command = CHROOT_CMD . 'rpmbuild ' . $build->parameters  . ' /usr/src/packages/SPECS/' . $rpm->getrpmspecname . ' 2>&1';
				
				$log->write($command,2);
				$output->write("<b>$command</b><br>");
				
				$temp_output = qx($command);
				
				$log->write($temp_output,3);
				$temp_output =~ s/\n/<br>/g;
				$output->write($temp_output);
				
				#connect and login to our ftp server
				$ftp = Net::FTP->new(SERVER_HOST);
				$ftp->login(SERVER_USERNAME,SERVER_PASSWORD);
		
				#set binary mode
				$ftp->binary;

				#get our status object so we can update accordingly
				($status) = $rpm->status;
				
				#upload our rpms
				if($temp_output =~ /Wrote:/g) {
                	#move the main rpm and srpm
					$ftp->put($file_dir . 'RPMS/i386/' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm',SERVER_BASE . $rpm->getrpmdir . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm');
					$output->write("<b>Uploaded RPM to Server</b>");
					$log->write('Uploaded ' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm' . ' to server',1);
					
					#retain our rpm information
					open(FH,$file_dir . 'RPMS/i386/' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm');
					binmode(FH);
					$md5->addfile(FH);
					close(FH);
					
					$rpminfo{'rpmsize'} = -s $file_dir . 'RPMS/i386/' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm';
					$rpminfo{'rpm_md5sum'} = $md5->hexdigest;
					
					$ftp->put($file_dir . 'SRPMS/' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.src.rpm',SERVER_BASE . $rpm->getsrpmdir . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.src.rpm');
					$output->write("<b>Uploaded SRPM to Server</b>");
					$log->write('Uploaded ' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.src.rpm' . ' to server',1);
					
					#retain our srpm information
					open(FH,$file_dir . 'SRPMS/' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.src.rpm');
					binmode(FH);
					$md5->addfile(FH);
					close(FH);
					
					$rpminfo{'srpmsize'} = -s $file_dir . 'SRPMS/' . $rpm->name . '-' . $rpm->version . '-' . $rpm->release . '.src.rpm';
					$rpminfo{'srpm_md5sum'} = $md5->hexdigest;
					
                	#also transfer any subpackages
					foreach($rpm->subpackages) {
						$ftp->put($file_dir . 'RPMS/i386/' . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm ',SERVER_BASE . $rpm->getrpmdir . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm ');
						$output->write("<b>Uploaded subpackage to the Server</b>");
						$log->write('Uploaded ' . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm to server',1);
						
						#update our subpackage database information
						open(FH,$file_dir . 'RPMS/i386/' . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm');
						binmode(FH);
						$md5->addfile(FH);
						close(FH);
						
						$_->set(
							rpmsize => -s $file_dir . 'RPMS/i386/' . $rpm->name . '-' . $_->name . '-' . $rpm->version . '-' . $rpm->release . '.i386.rpm',
							md5sum => $md5->hexdigest,
							);
						$_->update;
					}					
				
					#update the rpm status -- build successful
					$status->set(
								build => '0',
								success => '1',
								in_progress => '0',
								);
				} else {
					#update the rpm status -- error in build
					$status->set(
								build => '1',
								success => '0',
								in_progress => '0',
								);
				}
				
				#commit our database changes
				$status->update;
				
				#destroy our Log::LogLite object
				undef($output);
				
				#upload log to server
				$ftp->put('/var/tmp/' . $output_log,SERVER_BASE . 'build_logs/' . $rpm->sys_distribution_id->dirname . '/' . $output_log);
				$log->write("Uploaded $output_log to " . SERVER_BASE . 'build_logs/' . $rpm->sys_distribution_id->dirname . '/' . $output_log,1);
				
				#close the ftp connection
				$ftp->quit;

				#update our rpm information
				$rpminfo{'dateupdated'} = Time::Piece->new;
				$rpm->set(%rpminfo);
				$rpm->update;
				
				FAIL:
				#output $fail_message to log
				if($fail_message) {
					$output_log = 'FAIL: ' . $fail_message;
				}
				
				#update the buildqueue entry
				$queue->set(
							log_file => $output_log,
							start_time => $date,
							end_time => Time::Piece->new,
							);
				$queue->update;
			}
		}
		
		close READER;
		exit(1);
	}
}

exit(1);
